from typing import Optional

from pydantic import Field
from pyparsing import Iterator

from moonshot.src.datasets.dataset_arguments import (
    DatasetArguments as DatasetPydanticModel,
)


class DatasetCreateDTO(DatasetPydanticModel):
    id: Optional[str] = None
    examples: Iterator[dict] = None
    name: str = Field(..., min_length=1)
    description: str = Field(default="", min_length=1)
    license: Optional[str] = ""
    reference: Optional[str] = ""
    params: dict



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field


class SessionCreateDTO(BaseModel):
    model_config = ConfigDict(from_attributes=True)
    name: str = Field(min_length=3)
    description: Optional[str] = Field(default="", max_length=1000)
    endpoints: list[str] = Field(min_length=1)
    context_strategy: str = ""
    prompt_template: str = ""
    attack_module: str = ""
    metric: str = ""
    system_prompt: str = ""
    cs_num_of_prev_prompts: int = 5



from typing import Callable

from moonshot.src.runs.run_status import RunStatus


class RedTeamingProgress:
    DEFAULT_CHAT_BATCH_SIZE = 5

    def __init__(
        self,
        runner_id: str,
        red_teaming_arguments: dict,
        run_progress_callback_func: Callable | None,
    ):
        # Information on the run and callback for progress updating
        self.runner_id = runner_id
        self.chat_batch_size = red_teaming_arguments.get(
            "chat_batch_size", RedTeamingProgress.DEFAULT_CHAT_BATCH_SIZE
        )
        self.chats = red_teaming_arguments.get("chats", {})
        self.current_count = 0
        self.run_progress_callback_func = run_progress_callback_func
        self.status = RunStatus.PENDING

    def update_red_teaming_chats(
        self, red_teaming_prompt_arguments: dict, run_status: RunStatus
    ) -> None:
        """
        This method updates the red teaming chats with the provided arguments and run status.

        It calculates the response time by adding the duration to the start time. Then, it creates a dictionary
        with the prompt, response, prompt time, and response time. This dictionary is then added to the chats.

        Args:
            red_teaming_prompt_arguments (dict): A dictionary containing the arguments for the red teaming prompt.
            It should contain all keys in AttackModule.RedTeamingPromptArguments, which are mainly:
                - conn_id (str): The connection ID of the chat
                - cs_id (str): The context strategy ID (if any) or ""
                - pt_id (str): The prompt template ID (if any) or ""
                - am_id (str): The attack module ID
                - me_id (str): The metric ID (if any) or ""
                - original_prompt (str): The original prompt entered by the user
                - prepared_prompt(str): The modified and final prompt that was sent to the LLM
                - system_prompt (str): The system prompt entered by the user (if any)
                - response (str): The response from the LLM
                - duration (str): The amount of time it takes to get back the response from the LLM in seconds
                (in string)
                - start_time (str): The datetime of the prompt (in string)

            - run_status (RunStatus): The current status of the run.

        Returns:
            None
        """
        prompt_response_dict = {
            "conn_id": red_teaming_prompt_arguments["conn_id"],
            "context_strategy": red_teaming_prompt_arguments["cs_id"],
            "prompt_template": red_teaming_prompt_arguments["pt_id"],
            "attack_module": red_teaming_prompt_arguments["am_id"],
            "metric": red_teaming_prompt_arguments["me_id"],
            "prompt": red_teaming_prompt_arguments["original_prompt"],
            "prepared_prompt": red_teaming_prompt_arguments["prepared_prompt"],
            "system_prompt": red_teaming_prompt_arguments["system_prompt"],
            "predicted_result": red_teaming_prompt_arguments["response"],
            "duration": red_teaming_prompt_arguments["duration"],
            "prompt_time": red_teaming_prompt_arguments["start_time"],
        }

        if red_teaming_prompt_arguments["conn_id"] not in self.chats:
            self.chats[red_teaming_prompt_arguments["conn_id"]] = []
        self.chats[red_teaming_prompt_arguments["conn_id"]].append(prompt_response_dict)
        self.status = run_status

    def reset_chats(self) -> None:
        """
        This method clears all the chat data stored in the 'chats' attribute.
        """
        self.chats.clear()

    def update_red_teaming_progress(self) -> None:
        """
        This method updates the progress of the red teaming session.

        It checks if the current count of chats is equal to or greater than the batch size. If it is, it triggers
        a callback to notify the progress, resets the chats for the next batch, and resets the current count to zero.

        Regardless of the condition, it increments the current count by one.

        Args:
            None

        Returns:
            None
        """
        if self.current_count >= self.chat_batch_size:
            self.notify_progress()
            self.reset_chats()
            self.current_count = 0

        self.current_count += 1

    def notify_progress(self) -> None:
        """
        This method checks if a callback function for run progress exists and if so,
        it calls the function with the current state of the red teaming progress.
        """
        if self.run_progress_callback_func:
            self.run_progress_callback_func(self.get_dict())

    def get_dict(self) -> dict:
        """
        This method returns a dictionary representation of the current state of the red teaming progress.

        The dictionary includes the following keys:
        - "current_runner_id": The ID of the current runner.
        - "current_chats": The chats that will be returned during a callback.
        - "current_batch_size": The current batch size, which indicates the number of chats returned during a callback.
        - "current_status": The current status of the run.

        Returns:
            dict: A dictionary representation of the current state of the red teaming progress.
        """
        return {
            "current_runner_id": self.runner_id,
            "current_chats": self.chats,
            "current_batch_size": self.chat_batch_size,
            "current_status": self.status.name,
        }



from dependency_injector.wiring import inject

from ..schemas.benchmark_runner_dto import BenchmarkRunnerDTO
from ..services.base_service import BaseService
from ..services.benchmark_test_manager import BenchmarkTestManager
from ..services.utils.exceptions_handler import exception_handler
from ..types.types import BenchmarkCollectionType


class BenchmarkingService(BaseService):
    @inject
    def __init__(self, benchmark_test_manager: BenchmarkTestManager) -> None:
        self.benchmark_test_manager = benchmark_test_manager

    @exception_handler
    async def execute_cookbook(self, cookbook_executor_data: BenchmarkRunnerDTO) -> str:
        id = await self.benchmark_test_manager.schedule_test_task(
            cookbook_executor_data, BenchmarkCollectionType.COOKBOOK
        )
        return id

    @exception_handler
    async def execute_recipe(self, recipe_executor_data: BenchmarkRunnerDTO):
        id = await self.benchmark_test_manager.schedule_test_task(
            recipe_executor_data, BenchmarkCollectionType.RECIPE
        )
        return id

    @exception_handler
    async def cancel_executor(self, executor_id: str) -> None:
        await self.benchmark_test_manager.cancel_task(executor_id)



import asyncio
from ast import literal_eval

import cmd2
from rich.console import Console
from rich.table import Table
from slugify import slugify

from moonshot.api import (
    api_create_cookbook,
    api_create_runner,
    api_delete_cookbook,
    api_get_all_cookbook,
    api_get_all_run,
    api_get_all_runner_name,
    api_load_runner,
    api_read_cookbook,
    api_read_recipes,
    api_update_cookbook,
)
from moonshot.integrations.cli.benchmark.recipe import (
    _display_view_grading_scale_format,
    _display_view_statistics_format,
)
from moonshot.integrations.cli.cli_errors import (
    ERROR_BENCHMARK_ADD_COOKBOOK_DESC_VALIDATION,
    ERROR_BENCHMARK_ADD_COOKBOOK_NAME_VALIDATION,
    ERROR_BENCHMARK_ADD_COOKBOOK_RECIPES_LIST_STR_VALIDATION,
    ERROR_BENCHMARK_ADD_COOKBOOK_RECIPES_VALIDATION,
    ERROR_BENCHMARK_DELETE_COOKBOOK_COOKBOOK_VALIDATION,
    ERROR_BENCHMARK_LIST_COOKBOOK_FIND_VALIDATION,
    ERROR_BENCHMARK_LIST_COOKBOOK_PAGINATION_VALIDATION,
    ERROR_BENCHMARK_LIST_COOKBOOK_PAGINATION_VALIDATION_1,
    ERROR_BENCHMARK_RUN_COOKBOOK_COOKBOOKS_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_COOKBOOKS_VALIDATION_1,
    ERROR_BENCHMARK_RUN_COOKBOOK_ENDPOINTS_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_ENDPOINTS_VALIDATION_1,
    ERROR_BENCHMARK_RUN_COOKBOOK_NAME_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_NO_RESULT,
    ERROR_BENCHMARK_RUN_COOKBOOK_NUM_OF_PROMPTS_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_RANDOM_SEED_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_RESULT_PROC_MOD_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_RUNNER_PROC_MOD_VALIDATION,
    ERROR_BENCHMARK_RUN_COOKBOOK_SYS_PROMPT_VALIDATION,
    ERROR_BENCHMARK_UPDATE_COOKBOOK_COOKBOOK_VALIDATION,
    ERROR_BENCHMARK_UPDATE_COOKBOOK_UPDATE_VALUES_VALIDATION,
    ERROR_BENCHMARK_UPDATE_COOKBOOK_UPDATE_VALUES_VALIDATION_1,
    ERROR_BENCHMARK_VIEW_COOKBOOK_COOKBOOK_VALIDATION,
)
from moonshot.integrations.cli.common.display_helper import display_view_list_format
from moonshot.integrations.cli.utils.process_data import filter_data

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def add_cookbook(args) -> None:
    """
    Add a new cookbook.

    This function creates a new cookbook with the specified parameters.
    It first converts the recipes argument from a string to a list using the literal_eval function from the ast module.
    Then, it calls the api_create_cookbook function from the moonshot.api module to create the new cookbook.

    Args:
        args: A namespace object from argparse. It should have the following attributes:
            name (str): The name of the new cookbook.
            description (str): The description of the cookbook.
            recipes (str): A string representation of a list of recipes. Each recipe is represented by its ID.

    Raises:
        TypeError: If the 'name', 'description', or 'recipes' arguments are not strings or are None.
        ValueError: If the 'recipes' argument is not a list after evaluation.

    Returns:
        None
    """
    try:
        if not isinstance(args.name, str) or not args.name or args.name is None:
            raise TypeError(ERROR_BENCHMARK_ADD_COOKBOOK_NAME_VALIDATION)

        if (
            not isinstance(args.description, str)
            or not args.description
            or args.description is None
        ):
            raise TypeError(ERROR_BENCHMARK_ADD_COOKBOOK_DESC_VALIDATION)

        if (
            not isinstance(args.recipes, str)
            or not args.recipes
            or args.recipes is None
        ):
            raise TypeError(ERROR_BENCHMARK_ADD_COOKBOOK_RECIPES_VALIDATION)

        recipes = literal_eval(args.recipes)
        if not (
            isinstance(recipes, list)
            and all(isinstance(recipe, str) for recipe in recipes)
        ):
            raise ValueError(ERROR_BENCHMARK_ADD_COOKBOOK_RECIPES_LIST_STR_VALIDATION)

        new_cookbook_id = api_create_cookbook(args.name, args.description, recipes)
        print(f"[add_cookbook]: Cookbook ({new_cookbook_id}) created.")
    except Exception as e:
        print(f"[add_cookbook]: {str(e)}")


def list_cookbooks(args) -> list | None:
    """
    List all available cookbooks.

    This function retrieves all available cookbooks by calling the api_get_all_cookbook function from the
    moonshot.api module. It then filters the retrieved cookbooks based on the provided 'find' keyword and
    'pagination' parameters, and displays the filtered cookbooks using the _display_cookbooks function.

    Args:
        args: A namespace object from argparse. It should have the following optional attributes:
            find (str): Optional field to find cookbook(s) with a keyword.
            pagination (str): Optional field to paginate cookbooks. It should be a string representation of a tuple
                              containing two integers (page number and page size).

    Raises:
        TypeError: If the 'find' or 'pagination' arguments are not strings or are None.
        ValueError: If the 'pagination' argument is not a tuple of two integers after evaluation.

    Returns:
        list | None: A list of filtered cookbooks or None if there is no result.
    """

    try:
        if args.find is not None:
            if not isinstance(args.find, str) or not args.find:
                raise TypeError(ERROR_BENCHMARK_LIST_COOKBOOK_FIND_VALIDATION)

        if args.pagination is not None:
            if not isinstance(args.pagination, str) or not args.pagination:
                raise TypeError(ERROR_BENCHMARK_LIST_COOKBOOK_PAGINATION_VALIDATION)
            try:
                pagination = literal_eval(args.pagination)
                if not (
                    isinstance(pagination, tuple)
                    and len(pagination) == 2
                    and all(isinstance(i, int) for i in pagination)
                ):
                    raise ValueError(
                        ERROR_BENCHMARK_LIST_COOKBOOK_PAGINATION_VALIDATION_1
                    )
            except (ValueError, SyntaxError):
                raise ValueError(ERROR_BENCHMARK_LIST_COOKBOOK_PAGINATION_VALIDATION_1)
        else:
            pagination = ()

        cookbooks_list = api_get_all_cookbook()
        keyword = args.find.lower() if args.find else ""

        if cookbooks_list:
            filtered_cookbooks_list = filter_data(cookbooks_list, keyword, pagination)
            if filtered_cookbooks_list:
                _display_cookbooks(filtered_cookbooks_list)
                return filtered_cookbooks_list

        console.print("[red]There are no cookbooks found.[/red]")
        return None

    except Exception as e:
        print(f"[list_cookbooks]: {str(e)}")
        return None


def view_cookbook(args) -> None:
    """
    View a specific cookbook.

    This function retrieves a specific cookbook by calling the api_read_cookbook function from the
    moonshot.api module using the cookbook ID provided in the args.
    It then displays the retrieved cookbook using the display_view_cookbook function.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            cookbook (str): The ID of the cookbook to view.

    Raises:
        TypeError: If the 'cookbook' argument is not a string or is None.

    Returns:
        None
    """
    try:
        if (
            not isinstance(args.cookbook, str)
            or not args.cookbook
            or args.cookbook is None
        ):
            raise TypeError(ERROR_BENCHMARK_VIEW_COOKBOOK_COOKBOOK_VALIDATION)

        cookbook_info = api_read_cookbook(args.cookbook)
        _display_view_cookbook(cookbook_info)

    except Exception as e:
        print(f"[view_cookbook]: {str(e)}")


def run_cookbook(args) -> None:
    """
    Run a cookbook with the specified parameters.

    This function executes a cookbook runner with the given name, cookbooks, endpoints, and other parameters.
    It checks if the runner with the specified name already exists, and if not, it creates a new one.
    The cookbooks are run against the specified endpoints, and the results are processed and displayed.

    Args:
        args: A namespace object from argparse. It should have the following attributes:
            name (str): The name of the cookbook runner.
            cookbooks (str): A string representation of a list of cookbooks to run.
            endpoints (str): A string representation of a list of endpoints to run.
            num_of_prompts (int): The number of prompts to run.
            random_seed (int): The random seed number for reproducibility.
            system_prompt (str): The system prompt to use.
            runner_proc_module (str): The runner processing module to use.
            result_proc_module (str): The result processing module to use.

    Raises:
        TypeError: If any of the required arguments are not of the expected type or are None.
        ValueError: If the 'cookbooks' or 'endpoints' arguments are not lists of strings after evaluation.
        RuntimeError: If no results are found after running the cookbooks.

    Returns:
        None
    """
    try:
        if not isinstance(args.name, str) or not args.name or args.name is None:
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_NAME_VALIDATION)

        if (
            not isinstance(args.cookbooks, str)
            or not args.cookbooks
            or args.cookbooks is None
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_COOKBOOKS_VALIDATION)

        if (
            not isinstance(args.endpoints, str)
            or not args.endpoints
            or args.endpoints is None
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_ENDPOINTS_VALIDATION)

        if isinstance(args.num_of_prompts, bool) or not isinstance(
            args.num_of_prompts, int
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_NUM_OF_PROMPTS_VALIDATION)

        if isinstance(args.random_seed, bool) or not isinstance(args.random_seed, int):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_RANDOM_SEED_VALIDATION)

        if (
            not isinstance(args.system_prompt, str)
            or not args.system_prompt
            or args.system_prompt is None
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_SYS_PROMPT_VALIDATION)

        if (
            not isinstance(args.runner_proc_module, str)
            or not args.runner_proc_module
            or args.runner_proc_module is None
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_RUNNER_PROC_MOD_VALIDATION)

        if (
            not isinstance(args.result_proc_module, str)
            or not args.result_proc_module
            or args.result_proc_module is None
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_RESULT_PROC_MOD_VALIDATION)

        cookbooks = literal_eval(args.cookbooks)
        if not (
            isinstance(cookbooks, list)
            and all(isinstance(item, str) for item in cookbooks)
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_COOKBOOKS_VALIDATION_1)

        endpoints = literal_eval(args.endpoints)
        if not (
            isinstance(endpoints, list)
            and all(isinstance(item, str) for item in endpoints)
        ):
            raise TypeError(ERROR_BENCHMARK_RUN_COOKBOOK_ENDPOINTS_VALIDATION_1)

        # Run the cookbooks with the defined endpoints
        slugify_id = slugify(args.name, lowercase=True)
        if slugify_id in api_get_all_runner_name():
            cb_runner = api_load_runner(slugify_id)
        else:
            cb_runner = api_create_runner(args.name, endpoints)

        async def run():
            await cb_runner.run_cookbooks(
                cookbooks,
                args.num_of_prompts,
                args.random_seed,
                args.system_prompt,
                args.runner_proc_module,
                args.result_proc_module,
            )
            await cb_runner.close()

        loop = asyncio.get_event_loop()
        loop.run_until_complete(run())

        # Display results
        runner_runs = api_get_all_run(cb_runner.id)
        result_info = runner_runs[-1].get("results")
        if result_info:
            _show_cookbook_results(
                cookbooks, endpoints, result_info, result_info["metadata"]["duration"]
            )
        else:
            raise RuntimeError(ERROR_BENCHMARK_RUN_COOKBOOK_NO_RESULT)

    except Exception as e:
        print(f"[run_cookbook]: {str(e)}")


def update_cookbook(args) -> None:
    """
    Update a specific cookbook.

    This function updates a specific cookbook by calling the api_update_cookbook function using the
    cookbook name and update values provided in the args.

    Args:
        args: A namespace object from argparse. It should have the following attributes:
            cookbook (str): The id of the cookbook to update.
            update_values (str): A string representation of a list of tuples. Each tuple contains a key
            and a value to update in the cookbook.

    Raises:
        ValueError: If the 'cookbook' or 'update_values' arguments are not of the expected type or are None.

    Returns:
        None
    """
    try:
        if (
            args.cookbook is None
            or not isinstance(args.cookbook, str)
            or not args.cookbook
        ):
            raise ValueError(ERROR_BENCHMARK_UPDATE_COOKBOOK_COOKBOOK_VALIDATION)

        if (
            args.update_values is None
            or not isinstance(args.update_values, str)
            or not args.update_values
        ):
            raise ValueError(ERROR_BENCHMARK_UPDATE_COOKBOOK_UPDATE_VALUES_VALIDATION)

        cookbook = args.cookbook
        if literal_eval(args.update_values) and all(
            isinstance(i, tuple) for i in literal_eval(args.update_values)
        ):
            update_values = dict(literal_eval(args.update_values))
        else:
            raise ValueError(ERROR_BENCHMARK_UPDATE_COOKBOOK_UPDATE_VALUES_VALIDATION_1)
        api_update_cookbook(cookbook, **update_values)

        print("[update_cookbook]: Cookbook updated.")
    except Exception as e:
        print(f"[update_cookbook]: {str(e)}")


def delete_cookbook(args) -> None:
    """
    Delete a cookbook.

    This function deletes a cookbook with the specified identifier. It prompts the user for confirmation before
    proceeding with the deletion. If the user confirms, it calls the api_delete_cookbook function from the moonshot.api
    module to delete the cookbook. If the deletion is successful, it prints a confirmation message.

    If an exception occurs, it prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            cookbook (str): The identifier of the cookbook to delete.

    Raises:
        ValueError: If the 'cookbook' argument is not a string or is None.

    Returns:
        None
    """
    # Confirm with the user before deleting a cookbook
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the cookbook (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Cookbook deletion cancelled.[/]")
        return

    try:
        if (
            args.cookbook is None
            or not isinstance(args.cookbook, str)
            or not args.cookbook
        ):
            raise ValueError(ERROR_BENCHMARK_DELETE_COOKBOOK_COOKBOOK_VALIDATION)

        api_delete_cookbook(args.cookbook)
        print("[delete_cookbook]: Cookbook deleted.")
    except Exception as e:
        print(f"[delete_cookbook]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def _display_cookbooks(cookbooks_list):
    """
    Display the list of cookbooks in a tabular format.

    This function takes a list of cookbook dictionaries and displays each cookbook's details in a table.
    The table includes the cookbook's ID, name, description, and associated recipes. If the list is empty,
    it prints a message indicating that no cookbooks are found.

    Args:
        cookbooks_list (list): A list of dictionaries, where each dictionary contains the details of a cookbook.

    Returns:
        None
    """
    table = Table(
        title="List of Cookbooks", show_lines=True, expand=True, header_style="bold"
    )
    table.add_column("No.", width=2)
    table.add_column("Cookbook", justify="left", width=78)
    table.add_column("Contains", justify="left", width=20, overflow="fold")
    for idx, cookbook in enumerate(cookbooks_list, 1):
        id, name, description, recipes, *other_args = cookbook.values()
        idx = cookbook.get("idx", idx)
        cookbook_info = f"[red]ID: {id}[/red]\n\n[blue]{name}[/blue]\n{description}"
        recipes_info = display_view_list_format("Recipes", recipes)
        table.add_section()
        table.add_row(str(idx), cookbook_info, recipes_info)
    console.print(table)


def _display_view_cookbook(cookbook_info):
    """
    Display the cookbook information in a formatted table.

    This function takes a dictionary containing cookbook information and displays it in a table format using the rich
    library's Table class. It includes details such as the cookbook's ID, name, description, and associated recipes.

    Args:
        cookbook_info (dict): A dictionary containing the cookbook's information with keys such as
        'id', 'name', 'description', and 'recipes'.

    Returns:
        None
    """
    id, name, description, recipes = cookbook_info.values()
    recipes_list = api_read_recipes(recipes)
    if recipes_list:
        table = Table(
            title="View Cookbook", show_lines=True, expand=True, header_style="bold"
        )
        table.add_column("No.", width=2)
        table.add_column("Recipe", justify="left", width=78)
        table.add_column("Contains", justify="left", width=20, overflow="fold")
        for recipe_id, recipe in enumerate(recipes_list, 1):
            (
                id,
                name,
                description,
                tags,
                categories,
                datasets,
                prompt_templates,
                metrics,
                attack_strategies,
                grading_scale,
                stats,
            ) = recipe.values()

            tags_info = display_view_list_format("Tags", tags)
            categories_info = display_view_list_format("Categories", categories)
            datasets_info = display_view_list_format("Datasets", datasets)
            prompt_templates_info = display_view_list_format(
                "Prompt Templates", prompt_templates
            )
            metrics_info = display_view_list_format("Metrics", metrics)
            attack_strategies_info = display_view_list_format(
                "Attack Strategies", attack_strategies
            )
            grading_scale_info = _display_view_grading_scale_format(
                "Grading Scale", grading_scale
            )
            stats_info = _display_view_statistics_format("Statistics", stats)

            recipe_info = (
                f"[red]id: {id}[/red]\n\n[blue]{name}[/blue]\n{description}\n\n"
                f"{tags_info}\n\n{categories_info}\n\n{grading_scale_info}\n\n{stats_info}"
            )
            contains_info = f"{datasets_info}\n\n{prompt_templates_info}\n\n{metrics_info}\n\n{attack_strategies_info}"

            table.add_section()
            table.add_row(str(recipe_id), recipe_info, contains_info)
        console.print(table)
    else:
        console.print("[red]There are no recipes found for the cookbook.[/red]")


def _show_cookbook_results(cookbooks, endpoints, cookbook_results, duration):
    """
    Show the results of the cookbook benchmarking.

    This function takes the cookbooks, endpoints, cookbook results, and duration as arguments.
    If there are results, it generates a table with the cookbook results and prints a message indicating
    where the results are saved. If there are no results, it prints a message indicating that no results were found.
    Finally, it prints the duration of the run.

    Args:
        cookbooks (list): A list of cookbooks.
        endpoints (list): A list of endpoints.
        cookbook_results (dict): A dictionary with the results of the cookbook benchmarking.
        duration (float): The duration of the run.

    Returns:
        None
    """
    if cookbook_results:
        # Display recipe results
        _generate_cookbook_table(cookbooks, endpoints, cookbook_results)
    else:
        console.print("[red]There are no results.[/red]")

    # Print run stats
    run_stats = f"""{'='*50}\n[blue]Time taken to run: {duration}s[/blue]\n*Overall rating will be the lowest grade
     that the recipes have in each cookbook\n{'='*50}"""
    console.print(run_stats)


def _generate_cookbook_table(cookbooks: list, endpoints: list, results: dict) -> None:
    """
    Generate and display a table with the cookbook benchmarking results.

    This function creates a table that includes the index, cookbook name, recipe name, and the results
    for each endpoint.

    The cookbook names are prefixed with "Cookbook:" and are displayed with their overall grades. Each recipe under a
    cookbook is indented and prefixed with "Recipe:" followed by its individual grades for each endpoint. If there are
    no results for a cookbook, a row with dashes across all endpoint columns is added to indicate this.

    Args:
        cookbooks (list): A list of cookbook names to display in the table.
        endpoints (list): A list of endpoints for which results are to be displayed.
        results (dict): A dictionary containing the benchmarking results for cookbooks and recipes.

    Returns:
        None: The function prints the table to the console but does not return any value.
    """
    table = Table(
        title="Cookbook Result", show_lines=True, expand=True, header_style="bold"
    )
    table.add_column("No.", width=2)
    table.add_column("Cookbook (with its recipes)", justify="left", width=78)
    for endpoint in endpoints:
        table.add_column(endpoint, justify="center")

    index = 1
    for cookbook in cookbooks:
        # Get cookbook result
        cookbook_result = next(
            (
                result
                for result in results["results"]["cookbooks"]
                if result["id"] == cookbook
            ),
            None,
        )

        if cookbook_result:
            # Add the cookbook name with the "Cookbook: " prefix as the first row for this section
            endpoint_results = []
            for endpoint in endpoints:
                # Find the evaluation summary for the endpoint
                evaluation_summary = next(
                    (
                        temp_eval
                        for temp_eval in cookbook_result["overall_evaluation_summary"]
                        if temp_eval["model_id"] == endpoint
                    ),
                    None,
                )

                # Get the grade from the evaluation_summary, or use "-" if not found
                grade = "-"
                if evaluation_summary and evaluation_summary["overall_grade"]:
                    grade = evaluation_summary["overall_grade"]
                endpoint_results.append(grade)
            table.add_row(
                str(index),
                f"Cookbook: [blue]{cookbook}[/blue]",
                *endpoint_results,
                end_section=True,
            )

            for recipe in cookbook_result["recipes"]:
                endpoint_results = []
                for endpoint in endpoints:
                    # Find the evaluation summary for the endpoint
                    evaluation_summary = next(
                        (
                            temp_eval
                            for temp_eval in recipe["evaluation_summary"]
                            if temp_eval["model_id"] == endpoint
                        ),
                        None,
                    )

                    # Get the grade from the evaluation_summary, or use "-" if not found
                    grade = "-"
                    if (
                        evaluation_summary
                        and "grade" in evaluation_summary
                        and "avg_grade_value" in evaluation_summary
                        and evaluation_summary["grade"]
                    ):
                        grade = f"{evaluation_summary['grade']} [{evaluation_summary['avg_grade_value']}]"
                    endpoint_results.append(grade)

                # Add the recipe name indented under the cookbook name
                table.add_row(
                    "",
                    f"  └──  Recipe: [blue]{recipe['id']}[/blue]",
                    *endpoint_results,
                    end_section=True,
                )

            # Increment index only after all recipes of the cookbook have been added
            index += 1
        else:
            # If no results for the cookbook, add a row indicating this with the "Cookbook: " prefix
            # and a dash for each endpoint column
            table.add_row(
                str(index),
                f"Cookbook: {cookbook}",
                *(["-"] * len(endpoints)),
                end_section=True,
            )
            index += 1

    # Display table
    console.print(table)


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# Add cookbook arguments
add_cookbook_args = cmd2.Cmd2ArgumentParser(
    description="Add a new cookbook. The 'name' argument will be slugified to create a unique identifier.",
    epilog="Example:\n add_cookbook 'My new cookbook' "
    "'I am cookbook description' "
    "\"['analogical-similarity','auto-categorisation']\"",
)
add_cookbook_args.add_argument("name", type=str, help="Name of the new cookbook")
add_cookbook_args.add_argument(
    "description", type=str, help="Description of the new cookbook"
)
add_cookbook_args.add_argument(
    "recipes", type=str, help="List of recipes to be included in the new cookbook"
)

# Update cookbook arguments
update_cookbook_args = cmd2.Cmd2ArgumentParser(
    description="Update a cookbook.",
    epilog="Available keys for updating a cookbook: \n"
    "  name: The name of the cookbook. \n"
    "  description: The description of the cookbook. \n"
    "  recipes: A list of recipes included in the cookbook. \n\n"
    "Example command:\n"
    "  update_cookbook my-new-cookbook "
    "\"[('name', 'Updated Cookbook Name'), ('description', 'Updated description'), "
    "('recipes', ['analogical-similarity'])]\"",
)
update_cookbook_args.add_argument("cookbook", type=str, help="Id of the cookbook")
update_cookbook_args.add_argument(
    "update_values", type=str, help="Update cookbook key/value"
)

# View cookbook arguments
view_cookbook_args = cmd2.Cmd2ArgumentParser(
    description="View a cookbook.",
    epilog="Example:\n view_cookbook my-new-cookbook",
)
view_cookbook_args.add_argument("cookbook", type=str, help="Id of the cookbook")

# Delete cookbook arguments
delete_cookbook_args = cmd2.Cmd2ArgumentParser(
    description="Delete a cookbook.",
    epilog="Example:\n delete_cookbook my-new-cookbook",
)
delete_cookbook_args.add_argument("cookbook", type=str, help="Id of the cookbook")

# Run cookbook arguments
run_cookbook_args = cmd2.Cmd2ArgumentParser(
    description="Run a cookbook.",
    epilog="Example:\n run_cookbook "
    '"my new cookbook runner" '
    "\"['chinese-safety-cookbook']\" "
    "\"['openai-gpt35-turbo']\" "
    '-n 1 -r 1 -s "You are an intelligent AI" ',
)
run_cookbook_args.add_argument("name", type=str, help="Name of cookbook runner")
run_cookbook_args.add_argument("cookbooks", type=str, help="List of cookbooks to run")
run_cookbook_args.add_argument("endpoints", type=str, help="List of endpoints to run")
run_cookbook_args.add_argument(
    "-n", "--num_of_prompts", type=int, default=0, help="Number of prompts to run"
)
run_cookbook_args.add_argument(
    "-r", "--random_seed", type=int, default=0, help="Random seed number"
)
run_cookbook_args.add_argument(
    "-s", "--system_prompt", type=str, default="", help="System Prompt to use"
)
run_cookbook_args.add_argument(
    "-l",
    "--runner_proc_module",
    type=str,
    default="benchmarking",
    help="Runner processing module to use",
)
run_cookbook_args.add_argument(
    "-o",
    "--result_proc_module",
    type=str,
    default="benchmarking-result",
    help="Result processing module to use",
)

# List cookbook arguments
list_cookbooks_args = cmd2.Cmd2ArgumentParser(
    description="List all cookbooks.",
    epilog='Example:\n list_cookbooks -f "risk"',
)

list_cookbooks_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find cookbook(s) with keyword",
    nargs="?",
)

list_cookbooks_args.add_argument(
    "-p",
    "--pagination",
    type=str,
    help="Optional tuple to paginate cookbook(s). E.g. (2,10) returns 2nd page with 10 items in each page.",
    nargs="?",
)



from dependency_injector.wiring import inject

from moonshot.src.api.api_session import api_create_session
from moonshot.src.runners.runner import Runner

from .... import api as moonshot_api
from ..schemas.prompt_response_model import PromptResponseModel
from ..schemas.session_create_dto import SessionCreateDTO
from ..schemas.session_prompt_dto import SessionPromptDTO
from ..schemas.session_response_model import SessionMetadataModel, SessionResponseModel
from ..services.auto_red_team_test_manager import AutoRedTeamTestManager
from ..services.runner_service import RunnerService
from ..services.utils.exceptions_handler import exception_handler
from ..status_updater.interface.redteam_progress_callback import (
    InterfaceRedTeamProgressCallback,
)
from .base_service import BaseService


class SessionService(BaseService):
    active_runner: Runner

    @inject
    def __init__(
        self,
        auto_red_team_test_manager: AutoRedTeamTestManager,
        progress_status_updater: InterfaceRedTeamProgressCallback,
        runner_service: RunnerService,
    ) -> None:
        """
        Initialize the SessionService with dependencies.

        Args:
            progress_status_updater (InterfaceRedTeamProgressCallback): The callback interface for progress updates.
            runner_service (RunnerService): The service for managing runners.
        """
        self.auto_red_team_test_manager = auto_red_team_test_manager
        self.progress_status_updater = progress_status_updater
        self.runner_service = runner_service
        super().__init__()

    @exception_handler
    def create_new_session(
        self, session_create_dto: SessionCreateDTO
    ) -> SessionResponseModel:
        """
        Create a new session with a new runner and return the session metadata.

        Args:
            session_create_dto (SessionCreateDTO): Data transfer object containing session creation details.

        Returns:
            SessionMetadataModel: The metadata of the newly created session.
        """
        # Create a new runner instance using the runner service
        runner = self.runner_service.create_runner(
            runner_name=session_create_dto.name,
            endpoints=session_create_dto.endpoints,
            description=session_create_dto.description,
            progress_callback_func=self.progress_status_updater.on_art_progress_update,
        )
        self.active_runner = runner

        # Prepare runner arguments for session creation
        runner_args = {
            "context_strategy": session_create_dto.context_strategy,
            "prompt_template": session_create_dto.prompt_template,
            "cs_num_of_prev_prompts": session_create_dto.cs_num_of_prev_prompts,
            "attack_module": session_create_dto.attack_module,
            "metric": session_create_dto.metric,
            "system_prompt": session_create_dto.system_prompt,
        }

        # Create a new session if the runner has a database instance
        if runner.database_instance:
            api_create_session(
                runner.id, runner.database_instance, runner.endpoints, runner_args
            )

        # Load and return the session metadata
        session_metadata_dict = moonshot_api.api_load_session(runner.id)
        if session_metadata_dict is None:
            raise ValueError(f"No session metadata found for runner ID {runner.id}")

        return SessionResponseModel(
            session_name=runner.name,
            session_description=runner.description,
            session=SessionMetadataModel(**session_metadata_dict),
            chat_records=None,
        )

    @exception_handler
    def get_all_session(self) -> list[SessionMetadataModel]:
        """
        Retrieve all session metadata.

        Returns:
            list[SessionMetadataModel]: A list of session metadata models for all sessions.
        """
        retn_session = []
        runners_with_session = moonshot_api.api_get_all_runner()
        sessions_metadata_dicts = moonshot_api.api_get_all_session_metadata()
        
        runners_dict = {runner['id']: runner for runner in runners_with_session}
        
        for session in sessions_metadata_dicts:
            sess_id = session.get("session_id")
            if sess_id in runners_dict:
                session['description'] = runners_dict[sess_id]['description']
                retn_session.append(session)
        
        return [
            SessionMetadataModel(**metadata) for metadata in sessions_metadata_dicts
        ]

    @exception_handler
    def get_all_sessions_names(self) -> list[str]:
        """
        Retrieve a list of all session names.

        This method calls the moonshot API to get the names of all sessions and returns them.

        Returns:
            list[str]: A list of session names.
        """
        sessions = moonshot_api.api_get_all_session_names()
        return sessions

    @exception_handler
    def get_session_by_runner_id(
        self, runner_id: str, include_history: bool
    ) -> SessionResponseModel:
        """
        Retrieve session information by runner ID, optionally including chat history.

        Args:
            runner_id (str): The unique identifier of the runner.
            include_history (bool): Flag to determine if chat history should be included.

        Returns:
            SessionResponseModel: An object containing session metadata and optionally chat records.
        """
        runner: Runner = self.runner_service.load_runner(
            runner_id, self.progress_status_updater.on_art_progress_update
        )
        self.active_runner = runner
        session_metadata_dict = moonshot_api.api_load_session(self.active_runner.id)

        if not isinstance(session_metadata_dict, dict):
            raise ValueError(
                f"Session metadata for runner ID {runner.id} must be a dictionary."
            )

        session_chat = (
            moonshot_api.api_get_all_chats_from_session(runner.id)
            if include_history
            else None
        )

        return SessionResponseModel(
            session_name=runner.name,
            session_description=runner.description,
            session=SessionMetadataModel(**session_metadata_dict),
            chat_records=session_chat,
        )

    @exception_handler
    def update_session_chat(self, runner_id: str):
        """
        Update the chat for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.

        Returns:
            The updated chat records for the session.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        session_chat = moonshot_api.api_get_all_chats_from_session(
            self.active_runner.id
        )
        return session_chat

    @exception_handler
    def delete_session(self, runner_id: str) -> None:
        """
        Delete a session by runner ID.

        This method calls the moonshot API to delete the session associated with the given runner ID.

        Args:
            runner_id (str): The unique identifier of the runner whose session is to be deleted.
        """
        moonshot_api.api_delete_session(runner_id)

    @exception_handler
    def select_prompt_template(self, runner_id: str, prompt_template_name: str = ""):
        """
        Select a prompt template for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.
            prompt_template_name (str): The name of the prompt template to be selected.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        moonshot_api.api_update_prompt_template(
            self.active_runner.id, prompt_template_name
        )

    @exception_handler
    def select_ctx_strategy(
        self, runner_id: str, ctx_strategy_name: str = "", num_of_prompt: int = 5
    ) -> None:
        """
        Select a context strategy for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.
            ctx_strategy_name (str): The name of the context strategy to be selected.
            num_of_prompt (int): The number of previous prompts to consider in the strategy.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        moonshot_api.api_update_context_strategy(
            self.active_runner.id, ctx_strategy_name
        )
        moonshot_api.api_update_cs_num_of_prev_prompts(
            self.active_runner.id, num_of_prompt
        )

    @exception_handler
    def select_attack_module(self, runner_id: str, attack_module_name: str = ""):
        """
        Select an attack module for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.
            attack_module_name (str): The name of the attack module to be selected.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        moonshot_api.api_update_attack_module(self.active_runner.id, attack_module_name)

    @exception_handler
    def select_metric(self, runner_id: str, metric_name: str = ""):
        """
        Select a metric for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.
            metric_name (str): The name of the metric to be selected.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        moonshot_api.api_update_metric(self.active_runner.id, metric_name)

    @exception_handler
    def update_system_prompt(self, runner_id: str, system_prompt: str = ""):
        """
        Update the system prompt for the current session.

        Args:
            runner_id (str): The unique identifier of the runner.
            system_prompt (str): The new system prompt to be set.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")
        moonshot_api.api_update_system_prompt(self.active_runner.id, system_prompt)

    @exception_handler
    async def send_prompt(
        self, runner_id: str, prompt: SessionPromptDTO, batch_size: int = 5
    ) -> PromptResponseModel | str:
        """
        Send a prompt to the runner for processing.

        Args:
            runner_id (str): The unique identifier of the runner.
            prompt (SessionPromptDTO): The prompt data transfer object containing user prompt details.

        Returns:
            The result of the red teaming operation initiated by the prompt.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")

        session_metadata = moonshot_api.api_load_session(self.active_runner.id)
        if session_metadata is None:
            raise RuntimeError(
                f"No session metadata found for runner ID: {self.active_runner.id}"
            )

        def get_metadata_value(key, default=""):
            return session_metadata.get(key, default)

        prompt_template = get_metadata_value("prompt_template")
        context_strategy = get_metadata_value("context_strategy")
        num_of_prev_prompts = get_metadata_value("cs_num_of_prev_prompts")
        attack_module = get_metadata_value("attack_module")
        system_prompt = get_metadata_value("system_prompt")
        metric = get_metadata_value("metric")

        rt_args = {
            "prompt": prompt.user_prompt,
            "system_prompt": system_prompt,
            "context_strategy_info": [
                {
                    "context_strategy_id": context_strategy,
                    "num_of_prev_prompts": num_of_prev_prompts,
                }
            ]
            if context_strategy
            else [],
            "prompt_template_ids": [prompt_template] if prompt_template else [],
        }

        if attack_module:
            rt_args["attack_module_id"] = attack_module
            rt_args["metric_ids"] = [metric] if metric else []
            id = await self.auto_red_team_test_manager.schedule_art_task(
                rt_args, self.active_runner, batch_size
            )
            return id
        else:
            response = await self.active_runner.run_red_teaming(
                {"manual_rt_args": rt_args}
            )
            return PromptResponseModel.model_validate(response)

    @exception_handler
    async def cancel_auto_redteam(self, runner_id: str):
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")

        await self.auto_red_team_test_manager.cancel_task(self.active_runner.id)

    @exception_handler
    async def end_session(self, runner_id: str):
        """
        End the current session.

        Args:
            runner_id (str): The unique identifier of the runner whose session is to be ended.
        """
        if self.active_runner.id != runner_id:
            raise RuntimeError("Active session and requested session do not match.")

        self.active_runner.close()



import logging

from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Body, Depends, HTTPException, Query

from ..container import Container
from ..schemas.prompt_response_model import PromptResponseModel
from ..schemas.session_create_dto import SessionCreateDTO
from ..schemas.session_prompt_dto import SessionPromptDTO
from ..schemas.session_response_model import SessionMetadataModel, SessionResponseModel
from ..services.session_service import SessionService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Red Teaming"])
logger = logging.getLogger(__name__)


@router.get("/")
@inject
async def healthcheck() -> dict[str, str]:
    """
    Check the health of the web API.

    Returns:
        Dict[str, str]: The status message indicating the API is running.
    """
    return {"status": "web api is up and running"}


@router.get("/api/v1/sessions")
@inject
async def get_all_sessions(
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> list[SessionMetadataModel]:
    """
    Fetches a list of all session metadata from the service layer.

    This endpoint does not require any parameters and will return a list of session metadata objects.
    Each object contains details about a specific session without including the session history.

    Returns:
        List[SessionMetadataModel]: A list of session metadata objects.

    Raises:
        HTTPException: 404 error if no sessions are found.
                       400 error if there is a validation issue with the request.
                       500 error for any other server-side issues.
    """
    try:
        return session_service.get_all_session()

    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.get("/api/v1/sessions/name")
@inject
async def get_all_sessions_name(
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> list[str]:
    """
    Retrieve the names of all sessions.

    Args:
        session_service (SessionService): The service responsible for session operations.

    Returns:
        List[str]: A list of session names.

    Raises:
        HTTPException: An error with status code 404 if no session names are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return session_service.get_all_sessions_names()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.get("/api/v1/sessions/{runner_id}")
@inject
async def get_session_by_runner_id(
    runner_id: str,
    include_history: bool = Query(
        default=False,
        description="Flag to determine if session history should be included.",
    ),
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> SessionResponseModel:
    """
    Retrieve session data for a given runner ID, optionally including chat history.

    Args:
        runner_id (str): The unique identifier for the runner.
        include_history (bool): A flag to determine if the session history should be included in the response.
        session_service (SessionService): The service responsible for session operations.

    Returns:
        SessionResponseModel: The session data, including metadata and optionally chat records.

    Raises:
        HTTPException: An error with status code 404 if no session is found for the runner ID.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_data = session_service.get_session_by_runner_id(
            runner_id, include_history
        )
        return session_data
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.post("/api/v1/sessions")
@inject
async def create_session(
    session_dto: SessionCreateDTO,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> SessionResponseModel:
    """
    Create a new session based on the provided session data transfer object (DTO).

    Args:
        session_dto (SessionCreateDTO): The DTO containing the data needed to create a session.
        session_service (SessionService): The service responsible for session operations.

    Returns:
        SessionResponseModel: The metadata of the newly created session.

    Raises:
        HTTPException: An error with status code 404 if the session cannot be created due to a file not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        new_session = session_service.create_new_session(session_dto)
        return new_session
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.post("/api/v1/sessions/{runner_id}/prompt")
@inject
async def prompt(
    runner_id: str,
    user_prompt: SessionPromptDTO,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> PromptResponseModel | str:
    """
    Process a user prompt for a given session and return the session's response.

    This endpoint receives a prompt from the user and sends it to the specified session.
    The session's response to the prompt is then returned to the user.

    Args:
        runner_id (str): The unique identifier for the session to which the prompt will be sent.
        user_prompt (SessionPromptDTO): The data transfer object containing the prompt information from the user.
        session_service (SessionService): The service responsible for session management and prompt handling.

    Returns:
        PromptResponseModel: A model representing the response to the user's prompt
        ,including any chat records generated.

    Raises:
        HTTPException: Raised with status code 404 if the session associated with the runner_id is not found.
                       Raised with status code 400 if there is a validation error with the provided prompt data.
                       Raised with status code 500 for any other server-side errors encountered while processing.
    """
    try:
        result = await session_service.send_prompt(runner_id, user_prompt)
        return result
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.post("/api/v1/sessions/{runner_id}/cancel")
@inject
async def cancel_auto_redteam(
    runner_id: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
):
    """
    Cancel the automated red team operation for a given session.

    This endpoint is used to stop any ongoing automated red team operations for the session
    associated with the provided runner_id.

    Args:
        runner_id (str): The unique identifier for the session whose automated red team operation is to be canceled.
        session_service (SessionService): The service responsible for managing red team sessions.

    Raises:
        HTTPException: Raised with status code 404 if the session associated with the runner_id is not found.
                       Raised with status code 400 if there is a validation error with the runner_id.
                       Raised with status code 500 for any other server-side errors encountered while processing.
    """
    try:
        await session_service.cancel_auto_redteam(runner_id)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete("/api/v1/sessions/{session_id}")
@inject
async def delete_session(
    session_id: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Delete a session by its ID.

    Args:
        session_id (str): The unique identifier of the session to delete.
        session_service (SessionService): The service responsible for deleting sessions.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the session is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.delete_session(session_id)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.put("/api/v1/sessions/{runner_id}/prompt-template/{prompt_template_name}")
@inject
async def set_prompt_template(
    runner_id: str,
    prompt_template_name: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Select a prompt template for the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        prompt_template_name (str): The name of the prompt template to select.
        session_service (SessionService): The service responsible for managing session prompt templates.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the prompt template is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_prompt_template(runner_id, prompt_template_name)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete("/api/v1/sessions/{runner_id}/prompt-template/{prompt_template_name}")
@inject
async def unset_prompt_template(
    runner_id: str,
    prompt_template_name: str = "",
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Remove prompt template from the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        prompt_template_name (str): The name of the prompt template to remove.
        session_service (SessionService): The service responsible for managing session prompt templates.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the prompt template is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_prompt_template(runner_id, "")
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.put(
    "/api/v1/sessions/{runner_id}/context-strategy/{ctx_strategy_name}/{num_of_prompt}"
)
@inject
async def set_context_strategy(
    runner_id: str,
    ctx_strategy_name: str,
    num_of_prompt: int,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Set a context strategy for the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        ctx_strategy_name (str): The name of the context strategy to set.
        num_of_prompt (int): The number of prompts to apply the context strategy to.
        session_service (SessionService): The service responsible for managing session context strategies.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the context strategy is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_ctx_strategy(runner_id, ctx_strategy_name, num_of_prompt)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete(
    "/api/v1/sessions/{runner_id}/context-strategy/{ctx_strategy_name}/{num_of_prompt}"
)
@inject
async def unset_context_strategy(
    runner_id: str,
    ctx_strategy_name: str = "",
    num_of_prompt: int = 0,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Remove a context strategy from the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        ctx_strategy_name (str): The name of the context strategy to remove.
        num_of_prompt (int): The number of prompts to apply the context strategy to, defaults to 0.
        session_service (SessionService): The service responsible for managing session context strategies.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the context strategy is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_ctx_strategy(runner_id, "", 0)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.put("/api/v1/sessions/{runner_id}/attack-module/{atk_module_name}")
@inject
async def set_attack_module(
    runner_id: str,
    atk_module_name: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Set an attack module for the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        atk_module_name (str): The name of the attack module to set.
        session_service (SessionService): The service responsible for managing session attack modules.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the attack module is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_attack_module(runner_id, atk_module_name)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete("/api/v1/sessions/{runner_id}/attack-module/{atk_module_name}")
@inject
async def unset_attack_module(
    runner_id: str,
    atk_module_name: str = "",
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Remove an attack module from the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        atk_module_name (str): The name of the attack module to remove.
        session_service (SessionService): The service responsible for managing session attack modules.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the attack module is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_attack_module(runner_id, "")
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.put("/api/v1/sessions/{runner_id}/metric/{metric_name}")
@inject
async def set_metric(
    runner_id: str,
    metric_name: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Set a metric for the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        metric_name (str): The name of the metric to set.
        session_service (SessionService): The service responsible for managing session metrics.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the metric is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_metric(runner_id, metric_name)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete("/api/v1/sessions/{runner_id}/metric/{metric_name}")
@inject
async def unset_metric(
    runner_id: str,
    metric_name: str = "",
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    """
    Remove a metric from the current session.

    Args:
        runner_id (str): The unique identifier of the session.
        metric_name (str): The name of the metric to remove.
        session_service (SessionService): The service responsible for managing session metrics.

    Returns:
        Dict[str, bool]: A dictionary with a key 'success' indicating the operation result.

    Raises:
        HTTPException: An error with status code 404 if the metric is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        session_service.select_metric(runner_id, "")
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.put("/api/v1/sessions/{runner_id}/system-prompt")
@inject
async def set_system_prompts(
    runner_id: str,
    system_prompt: str = Body(..., embed=True),
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    try:
        session_service.update_system_prompt(runner_id, system_prompt)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.delete("/api/v1/sessions/{runner_id}/system-prompt")
@inject
async def unset_system_prompts(
    runner_id: str,
    system_prompt: str = "",
    session_service: SessionService = Depends(Provide[Container.session_service]),
) -> dict[str, bool]:
    try:
        session_service.update_system_prompt(runner_id, "")
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)


@router.get("/api/v1/sessions/{runner_id}/close")
@inject
async def close_session(
    runner_id: str,
    session_service: SessionService = Depends(Provide[Container.session_service]),
):
    try:
        await session_service.end_session(runner_id)
        return {"success": True}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(status_code=404, detail=e.msg)
        elif e.error_code == "ValidationError":
            raise HTTPException(status_code=400, detail=e.msg)
        else:
            raise HTTPException(status_code=500, detail=e.msg)



from .... import api as moonshot_api
from ..schemas.bookmark_create_dto import BookmarkCreateDTO, BookmarkPydanticModel
from ..services.base_service import BaseService
from ..services.utils.exceptions_handler import exception_handler
from .utils.file_manager import copy_file


class BookmarkService(BaseService):
    @exception_handler
    def insert_bookmark(self, bookmark_data: BookmarkCreateDTO) -> dict:
        """
        Inserts a new bookmark into the system.

        Args:
            bookmark_data (BookmarkCreateDTO): The data transfer object containing bookmark details.
        """
        result = moonshot_api.api_insert_bookmark(
            name=bookmark_data.name,
            prompt=bookmark_data.prompt,
            prepared_prompt=bookmark_data.prepared_prompt,
            response=bookmark_data.response,
            context_strategy=bookmark_data.context_strategy,
            prompt_template=bookmark_data.prompt_template,
            attack_module=bookmark_data.attack_module,
            metric=bookmark_data.metric,
        )

        return result

    @exception_handler
    def get_all_bookmarks(self, name: str | None = None) -> list[BookmarkPydanticModel]:
        """
        Retrieves all bookmarks or a specific bookmark by its name.

        Args:
            name (str | None, optional): The name of the bookmark to retrieve. If None, all bookmarks are retrieved.

        Returns:
            list[BookmarkPydanticModel]: A list of bookmark models.
        """
        retn_bookmark: list[BookmarkPydanticModel] = []

        if name:
            bookmarks = [moonshot_api.api_get_bookmark(name)]
        else:
            bookmarks = moonshot_api.api_get_all_bookmarks()

        for bookmark in bookmarks:
            retn_bookmark.append(BookmarkPydanticModel(**bookmark))
        return retn_bookmark

    @exception_handler
    def delete_bookmarks(self, all: bool = False, name: str | None = None) -> dict:
        """
        Deletes a single bookmark by its name or all bookmarks if the 'all' flag is set to True and returns
        a boolean indicating the success of the operation.

        Args:
            all (bool, optional): If True, all bookmarks will be deleted. Defaults to False.
            name (str | None, optional): The name of the bookmark to delete. If 'all' is False, 'name' must be provided.

        Returns:
            dict: True if the deletion was successful, False otherwise.
        """
        if all:
            result = moonshot_api.api_delete_all_bookmark()
        elif name is not None:
            result = moonshot_api.api_delete_bookmark(name)
        else:
            raise ValueError("Either 'all' must be True or 'name' must be provided.")

        if not result["success"]:
            raise Exception(
                result["message"], "delete_bookmarks", "DeleteBookmarkError"
            )

        return result

    @exception_handler
    def export_bookmarks(self, export_file_name: str = "bookmarks") -> str:
        """
        Exports bookmarks to a file or returns them as a list of dictionaries.

        Args:
            write_file (bool, optional): If True, bookmarks will be written to a file. Defaults to False.
            export_file_name (str, optional): The name of the file to write bookmarks to. Defaults to "bookmarks".

        Returns:
            list[dict]: A list of bookmarks as dictionaries.
        """

        new_file_path = moonshot_api.api_export_bookmarks(export_file_name)

        return copy_file(new_file_path)



from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException, Query

from ..container import Container
from ..schemas.dataset_create_dto import DatasetCreateDTO
from ..schemas.dataset_response_dto import DatasetResponseDTO
from ..services.dataset_service import DatasetService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Datasets"])


@router.post("/api/v1/datasets")
@inject
def create_dataset(
    dataset_data: DatasetCreateDTO,
    method: str = Query(
        ...,
        description="The method to use for creating the dataset. Supported methods are 'hf' and 'csv'.",
    ),
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> str:
    """
    Create a new dataset using the specified method.

    Args:
        dataset_data (DatasetCreateDTO): The data required to create the dataset.
        method (str): The method to use for creating the dataset. Supported methods are "hf" and "csv".
        dataset_service (DatasetService, optional): The service responsible for creating the dataset.
        Defaults to Depends(Provide[Container.dataset_service]).

    Returns:
        dict: A message indicating the dataset was created successfully.

    Raises:
        HTTPException: An error with status code 404 if the dataset file is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.create_dataset(dataset_data, method)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve datasets: {e.msg}"
            )


@router.get("/api/v1/datasets")
@inject
def get_all_datasets(
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> list[DatasetResponseDTO]:
    """
    Retrieve all datasets from the database.

    Args:
        dataset_service (DatasetService): The service responsible for retrieving datasets.

    Returns:
        list: A list of all datasets.

    Raises:
        HTTPException: An error with status code 404 if no datasets are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.get_all_datasets()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve datasets: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve datasets: {e.msg}"
            )


@router.get("/api/v1/datasets/name")
@inject
def get_all_datasets_name(
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> list[str]:
    """
    Retrieve the names of all datasets from the database.

    Args:
        dataset_service (DatasetService): The service responsible for retrieving dataset names.

    Returns:
        list[str]: A list of dataset names.

    Raises:
        HTTPException: An error with status code 404 if no dataset names are found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        return dataset_service.get_all_datasets_name()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve dataset names: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve dataset names: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve dataset names: {e.msg}"
            )


@router.delete("/api/v1/datasets/{dataset_id}")
@inject
def delete_dataset(
    dataset_id: str,
    dataset_service: DatasetService = Depends(Provide[Container.dataset_service]),
) -> dict[str, str]:
    """
    Delete a dataset from the database by its ID.

    Args:
        dataset_id (str): The unique identifier of the dataset to delete.
        dataset_service (DatasetService): The service responsible for deleting the dataset.

    Returns:
        dict[str, str]: A message indicating the successful deletion of the dataset.

    Raises:
        HTTPException: An error with status code 404 if the dataset is not found.
                       An error with status code 400 if there is a validation error.
                       An error with status code 500 for any other server-side error.
    """
    try:
        dataset_service.delete_dataset(dataset_id)
        return {"message": "Dataset deleted successfully"}
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete dataset: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete dataset: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete dataset: {e.msg}"
            )



from abc import ABC, abstractmethod
from ...types.types import TestRunProgress

class InterfaceRedTeamProgressCallback(ABC):
    """
    The interface to implement for handling of redteam progress updates from moonshot library

    """
    @abstractmethod
    def on_art_progress_update(self, progress_data: TestRunProgress) -> None:
        pass






import asyncio
from typing import Any, TypedDict

from moonshot.integrations.web_api.services.base_service import BaseService
from moonshot.integrations.web_api.types.types import RedTeamTestProgress
from moonshot.src.runners.runner import Runner


class AutoRedTeamTaskInfo(TypedDict):
    async_task: asyncio.Task[Any]
    moonshot_runner: Runner
    status: RedTeamTestProgress | None


class AutoRedTeamTestState(BaseService):
    state: dict[str, AutoRedTeamTaskInfo] = {}

    def add_task(
        self, id: str, task: asyncio.Task[Any], moonshot_runner: Runner
    ) -> None:
        self.state[id] = {
            "async_task": task,
            "moonshot_runner": moonshot_runner,
            "status": None,
        }

    def remove_task(self, id: str) -> AutoRedTeamTaskInfo:
        return self.state.pop(id)

    async def cancel_task(self, id: str) -> None:
        task_info = self.state[id]
        if task_info and not task_info["async_task"].done():
            if task_info["moonshot_runner"]:
                await task_info["moonshot_runner"].cancel()
            task_info["async_task"].cancel()
            self.logger.debug(f"Task {id} has been cancelled")
        else:
            self.logger.debug(f"Task {id} is already completed or does not exist.")

    def update_progress_status(self, updates: RedTeamTestProgress) -> None:
        self.logger.debug(updates)
        self.state[updates["current_runner_id"]]["status"] = updates

    def get_state(self) -> dict[str, AutoRedTeamTaskInfo]:
        return self.state

    def get_progress_status(self, id: str) -> RedTeamTestProgress | None:
        if id in self.state:
            return self.state[id]["status"]
        return None

    def get_all_progress_status(self) -> dict[str, RedTeamTestProgress]:
        return {
            id: task_info["status"]
            for id, task_info in self.state.items()
            if task_info["status"] is not None
        }



from typing import Optional

from moonshot.src.cookbooks.cookbook_arguments import (
    CookbookArguments as CookbookPydanticModel,
)
from pydantic import Field

class CookbookCreateDTO(CookbookPydanticModel):
    id: Optional[str] = None
    name: str = Field(..., min_length=1)
    description: Optional[str] = Field(default="", min_length=1)
    recipes: list[str] = Field(..., min_length=1)


class CookbookUpdateDTO(CookbookPydanticModel):
    id: Optional[str] = None
    name: Optional[str] = Field(default=None, min_length=1)
    description: Optional[str] = Field(default=None, min_length=1)
    recipes: Optional[list[str]] = Field(default=None, min_length=1)


from typing import Optional

from moonshot.src.connectors_endpoints.connector_endpoint_arguments import (
    ConnectorEndpointArguments as ConnectorEndpointPydanticModel,
)


class EndpointCreateDTO(ConnectorEndpointPydanticModel):
    id: Optional[str] = None


class EndpointUpdateDTO(ConnectorEndpointPydanticModel):
    id: Optional[str] = None
    name: Optional[str] = None
    connector_type: Optional[str] = None
    uri: Optional[str] = None
    token: Optional[str] = None
    max_calls_per_second: Optional[int] = None
    max_concurrency: Optional[int] = None
    params: Optional[dict] = None
    created_date: Optional[str] = None



from pydantic import BaseModel
from pyparsing import Iterator


class DatasetArguments(BaseModel):
    class Config:
        arbitrary_types_allowed = True

    # id (str): Unique identifier for the dataset
    id: str

    # name (str): Name of the dataset
    name: str

    # description (str): Description of the dataset's contents and purpose
    description: str

    # examples (Iterator[dict] | None): Generator of examples from the dataset, where each example is a dictionary.
    examples: Iterator[dict] | None

    # num_of_dataset_prompts (int): The number of dataset prompts, automatically calculated
    num_of_dataset_prompts: int = 0

    # created_date (str): The creation date and time of the dataset in ISO format without 'T'. Automatically generated.
    created_date: str = ""

    # reference (str): An optional string to store a reference link or identifier for the dataset
    reference: str = ""

    # license (str): License information for the dataset. Defaults to an empty string if not provided.
    license: str = ""

    def to_dict(self) -> dict:
        """
        Converts the DatasetArguments object to a dictionary.

        Returns:
            dict: A dictionary representation of the DatasetArguments object, including the id, name, description,
                  examples, number of dataset prompts, created date, reference, and license.
        """
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "examples": self.examples,
            "num_of_dataset_prompts": self.num_of_dataset_prompts,
            "created_date": self.created_date,
            "reference": self.reference,
            "license": self.license,
        }



from moonshot.src.cookbooks.cookbook_arguments import CookbookArguments as Cookbook
from moonshot.src.recipes.recipe_arguments import RecipeArguments as Recipe

from .... import api as moonshot_api
from ..schemas.cookbook_create_dto import CookbookCreateDTO, CookbookUpdateDTO
from ..schemas.cookbook_response_model import CookbookResponseModel
from ..services.base_service import BaseService
from ..services.recipe_service import get_total_prompt_in_recipe
from ..services.utils.exceptions_handler import exception_handler


class CookbookService(BaseService):
    @exception_handler
    def create_cookbook(self, cookbook_data: CookbookCreateDTO) -> None:
        """
        Create a new cookbook with the given data.

        Args:
            cookbook_data (CookbookCreateDTO): Data transfer object containing cookbook details.
        """
        moonshot_api.api_create_cookbook(
            name=cookbook_data.name,
            description=cookbook_data.description,
            recipes=cookbook_data.recipes,
        )

    @exception_handler
    def get_all_cookbooks(
        self,
        tags: str,
        categories: str,
        count: bool,
        ids: str | None = None,
        categories_excluded: str | None = None,
    ) -> list[CookbookResponseModel]:
        """
        Retrieve all cookbooks, optionally filtered by tags and categories, and with prompt counts.

        Args:
            ids (str): Filter cookbooks by IDs.
            tags (str): Filter cookbooks by tags.
            categories (str): Filter cookbooks by categories.
            count (bool): Include the total prompt count in each cookbook.

        Returns:
            list[CookbookResponseModel]: A list of cookbook response models.
        """
        cookbooks_list: list[CookbookResponseModel] = []

        if ids:
            cookbook_ids = ids.split(",")
            cookbooks = [
                moonshot_api.api_read_cookbook(cookbook_id)
                for cookbook_id in cookbook_ids
            ]
        else:
            cookbooks = moonshot_api.api_get_all_cookbook()

        for cookbook_dict in cookbooks:
            cookbook = CookbookResponseModel(**cookbook_dict)

            if not tags and not categories:
                if cookbook not in cookbooks_list:
                    cookbooks_list.append(cookbook)
                    if count:
                        cookbook.total_prompt_in_cookbook = (
                            get_total_prompt_in_cookbook(cookbook)
                        )

            if tags and cookbooks_recipe_has_tags(tags, cookbook):
                if cookbook not in cookbooks_list:
                    cookbooks_list.append(cookbook)
                    if count:
                        cookbook.total_prompt_in_cookbook = (
                            get_total_prompt_in_cookbook(cookbook)
                        )

            if categories and cookbooks_recipe_has_categories(categories, cookbook):
                if cookbook not in cookbooks_list:
                    cookbooks_list.append(cookbook)
                    if count:
                        cookbook.total_prompt_in_cookbook = (
                            get_total_prompt_in_cookbook(cookbook)
                        )

            if categories_excluded and cookbooks_recipe_has_categories(
                categories_excluded, cookbook
            ):
                cookbooks_list.remove(cookbook)

        return cookbooks_list

    @exception_handler
    def get_all_cookbooks_names(self) -> list[str]:
        """
        Retrieve the names of all cookbooks.

        Returns:
            list[str]: A list of cookbook names.
        """
        cookbooks = moonshot_api.api_get_all_cookbook_name()
        return cookbooks

    @exception_handler
    def update_cookbook(
        self, cookbook_data: CookbookUpdateDTO, cookbook_id: str
    ) -> None:
        """
        Update an existing cookbook with new data.

        Args:
            cookbook_data (CookbookCreateDTO): Data transfer object containing new cookbook details.
            cookbook_id (str): The ID of the cookbook to update.
        """
        update_data = {
            k: v
            for k, v in cookbook_data.to_dict().items()
            if v is not None and k != "id"
        }
        moonshot_api.api_update_cookbook(cb_id=cookbook_id, **update_data)

    @exception_handler
    def delete_cookbook(self, cookbook_id: str) -> None:
        """
        Delete a cookbook by its ID.

        Args:
            cookbook_id (str): The ID of the cookbook to delete.
        """
        moonshot_api.api_delete_cookbook(cookbook_id)


@staticmethod
def get_total_prompt_in_cookbook(cookbook: Cookbook) -> int:
    """
    Calculate the total number of prompts in a cookbook.

    This function sums up the total prompts for each recipe in the cookbook.

    Args:
        cookbook (Cookbook): The cookbook object containing the recipe IDs.

    Returns:
        int: The total count of prompts within the cookbook.
    """
    recipes = moonshot_api.api_read_recipes(cookbook.recipes)
    return sum(get_total_prompt_in_recipe(Recipe(**recipe)) for recipe in recipes)


@staticmethod
def cookbooks_recipe_has_tags(tags: str, cookbook: Cookbook) -> bool:
    """
    Check if any recipe in a cookbook has the specified tags.

    Args:
        tags (str): The tags to check for in the cookbook's recipes.
        cookbook (Cookbook): The cookbook object containing the recipe IDs.

    Returns:
        bool: True if any recipe in the cookbook has the specified tags, False otherwise.
    """
    recipe_ids = cookbook.recipes
    recipes = moonshot_api.api_read_recipes(recipe_ids)
    for recipe in recipes:
        recipe = Recipe(**recipe)
        if tags in recipe.tags:
            return True
    return False


@staticmethod
def cookbooks_recipe_has_categories(categories: str, cookbook: Cookbook) -> bool:
    """
    Check if any recipe in a cookbook has the specified categories.

    Args:
        categories (str): The categories to check for in the cookbook's recipes.
        cookbook (Cookbook): The cookbook object containing the recipe IDs.
        exclude_categories (str): The categories to exclude

    Returns:
        bool: True if any recipe in the cookbook has the specified categories, False otherwise.
    """
    recipe_ids = cookbook.recipes
    categories_list = [category.lower() for category in categories.split(",")]
    recipes = moonshot_api.api_read_recipes(recipe_ids)
    for recipe in recipes:
        recipe = Recipe(**recipe)
        if any(
            category in [rcat.lower() for rcat in recipe.categories]
            for category in categories_list
        ):
            return True
    return False



from moonshot.src.api.api_runner import api_get_all_runner, api_load_runner
from moonshot.src.runs.run import Run
from moonshot.src.runs.run_arguments import RunArguments


# ------------------------------------------------------------------------------
# Run APIs
# ------------------------------------------------------------------------------
def api_get_all_run(runner_id: str = "") -> list[dict]:
    """
    Retrieves all runs for a given runner ID or for all runners if no ID is provided.

    This function calls an internal API to get available runs and then converts each run into a dictionary format.

    Args:
        runner_id (str, optional): The ID of the runner to retrieve runs for. If empty, runs for all runners
        are retrieved.

    Returns:
        list[dict]: A list of dictionaries, each representing a run's data.
    """
    _, runs = _api_get_available_runs(runner_id)
    return [run.to_dict() for run in runs]


def _api_get_available_runs(
    runner_id: str = "",
) -> tuple[list[str], list[RunArguments]]:
    """
    Retrieves available runs and their corresponding runner IDs.

    This function fetches information about available runs based on the provided runner ID. If no runner ID is
    provided, it fetches runs for all runners. It returns a tuple containing a list of runner IDs and a list of
    RunArguments instances representing the runs.

    Args:
        runner_id (str): The ID of the runner for which to retrieve run information. If empty, information for
                         all runners is retrieved.

    Returns:
        tuple[list[str], list[RunArguments]]: A tuple containing a list of runner IDs and a list of RunArguments
                                              instances for the corresponding runs.
    """
    # Lists to hold runner IDs and Run instances.
    runners_ids = []
    runs = []

    # Retrieve information for all runners.
    runners_info = api_get_all_runner()

    # Load the specified runner, or all runners if no ID is provided.
    if runner_id:
        runner_instances = (
            [api_load_runner(runner_id)]
            if any(runner_id == runner_info.get("id") for runner_info in runners_info)
            else []
        )
    else:
        runner_instances = [
            api_load_runner(str(runner_info.get("id"))) for runner_info in runners_info
        ]

    # Collect runs from each runner instance.
    for runner_instance in runner_instances:
        # Only proceed if the runner has an associated database instance.
        if runner_instance.database_instance:
            # Fetch all runs from the database associated with the runner.
            all_runs = Run.get_all_runs(runner_instance.database_instance)
            # Record the runner ID and the runs.
            runners_ids.append(runner_instance.id)
            runs.extend(all_runs)

    # Return the runner IDs and their runs.
    return runners_ids, runs



import os
from typing import Optional

from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException, Query
from fastapi.responses import FileResponse

from ..container import Container
from ..schemas.bookmark_create_dto import BookmarkCreateDTO, BookmarkPydanticModel
from ..services.bookmark_service import BookmarkService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Bookmark"])


@router.post(
    "/api/v1/bookmarks", response_description="Bookmark data added to the database"
)
@inject
def insert_bookmark(
    bookmark_data: BookmarkCreateDTO,
    bookmark_service: BookmarkService = Depends(Provide[Container.bookmark_service]),
) -> dict:
    """
    Insert a new bookmark into the database.

    Args:
        bookmark_data: The data of the bookmark to be added.
        bookmark_service: The service responsible for bookmark operations.

    Returns:
        A dictionary with a message indicating successful insertion.

    Raises:
        HTTPException: An error occurred while inserting the bookmark.
    """
    try:
        result = bookmark_service.insert_bookmark(bookmark_data)
        return result
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to insert bookmark: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to insert bookmark: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to insert bookmark: {e.msg}"
            )


@router.get(
    "/api/v1/bookmarks",
    response_description="List of all bookmarks or a specific bookmark by name",
)
@inject
def get_all_bookmarks(
    name: Optional[str] = Query(None, description="Name of the bookmark to query"),
    bookmark_service: BookmarkService = Depends(Provide[Container.bookmark_service]),
) -> list[BookmarkPydanticModel]:
    """
    Retrieve all bookmarks or a specific bookmark by name from the database.

    Args:
        name: The name of the bookmark to retrieve. If None, all bookmarks are retrieved.
        bookmark_service: The service responsible for bookmark operations.

    Returns:
        A list of bookmarks or a single bookmark if a name is provided.

    Raises:
        HTTPException: An error occurred while retrieving bookmarks.
    """
    try:
        return bookmark_service.get_all_bookmarks(
            name=name,
        )
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve bookmarks: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve bookmarks: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve bookmarks: {e.msg}"
            )


@router.delete(
    "/api/v1/bookmarks", response_description="Bookmark data deleted from the database"
)
@inject
def delete_bookmark(
    all: bool = Query(False, description="Flag to delete all bookmarks"),
    name: Optional[str] = Query(None, description="Name of the bookmark to delete"),
    bookmark_service: BookmarkService = Depends(Provide[Container.bookmark_service]),
) -> dict:
    """
    Delete a specific bookmark by name or all bookmarks from the database.

    Args:
        all: A flag indicating whether to delete all bookmarks.
        name: The name of the bookmark to delete. If 'all' is False and 'name' is None, no bookmark will be deleted.
        bookmark_service: The service responsible for bookmark operations.

    Returns:
        A dictionary with a message indicating successful deletion or an error message.

    Raises:
        HTTPException: An error occurred while deleting the bookmark(s).
    """
    try:
        if all:
            return bookmark_service.delete_bookmarks(all=True)
        elif name is not None:
            return bookmark_service.delete_bookmarks(all=False, name=name)
        else:
            raise HTTPException(
                status_code=400, detail="Must specify 'all' or 'name' parameter"
            )
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to delete bookmark: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to delete bookmark: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to delete bookmark: {e.msg}"
            )
@router.get(
    "/api/v1/bookmarks/export", response_description="Exporting Bookmark to JSON file"
)
@inject
def export_bookbookmarks(
    export_file_name: Optional[str] = Query(
        "bookmarks", description="Name of the exported file"
    ),
    bookmark_service: BookmarkService = Depends(Provide[Container.bookmark_service]),
) -> FileResponse:
    """
    Export bookmarks to a JSON file with a given file name.

    Args:
        export_file_name: The name of the file to export the bookmarks to.z
        bookmark_service: The service responsible for bookmark operations.

    Returns:
        A FileResponse with the path to the exported file or an error message.
    """
    try:
        file_path = bookmark_service.export_bookmarks(export_file_name)
        return FileResponse(file_path, media_type="application/json", filename=export_file_name)
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to export bookmark: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to export bookmark: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to export bookmark: {e.msg}"
            )



from pydantic import validate_call

from moonshot.src.datasets.dataset import Dataset
from moonshot.src.datasets.dataset_arguments import DatasetArguments


# ------------------------------------------------------------------------------
# Datasets APIs
# ------------------------------------------------------------------------------
@validate_call
def api_delete_dataset(ds_id: str) -> bool:
    """
    Deletes a dataset identified by its unique dataset ID.

    Args:
        ds_id (str): The unique identifier for the dataset to be deleted.

    Returns:
        bool: True if the dataset was successfully deleted.

    Raises:
        Exception: If the deletion process encounters an error.
    """
    return Dataset.delete(ds_id)


def api_get_all_datasets() -> list[dict]:
    """
    This function retrieves all available datasets and returns them as a list of dictionaries. Each dictionary
    represents a result and contains its information.

    Returns:
        list[dict]: A list of dictionaries, each representing a result.
    """
    _, datasets = Dataset.get_available_items()
    return [dataset.to_dict() for dataset in datasets]


def api_get_all_datasets_name() -> list[str]:
    """
    This function retrieves all available datasets names and returns them as a list.

    Returns:
        list[str]: A list of datasets names.
    """
    datasets_name, _ = Dataset.get_available_items()
    return datasets_name


def api_create_datasets(
    name: str, description: str, reference: str, license: str, method: str, **kwargs
) -> str:
    """
    This function creates a new dataset.

    This function takes the name, description, reference, and license for a new dataset as input. It then creates a new
    DatasetArguments object with these details and an empty id. The id is left empty because it will be generated
    from the name during the creation process. The function then calls the Dataset's create method to
    create the new dataset.

    Args:
        name (str): The name of the new dataset.
        description (str): A brief description of the new dataset.
        reference (str): A reference link for the new dataset.
        license (str): The license of the new dataset.
        method (str): The method to create new dataset. (csv/hf)
        kwargs: Additional keyword arguments for the Dataset's create method.

    Returns:
        str: The ID of the newly created dataset.
    """
    ds_args = DatasetArguments(
        id="",
        name=name,
        description=description,
        reference=reference,
        license=license,
        examples=None,
    )

    return Dataset.create(ds_args, method, **kwargs)



from dependency_injector.wiring import Provide, inject
from fastapi import APIRouter, Depends, HTTPException

from ..container import Container
from ..services.attack_module_service import AttackModuleService
from ..services.utils.exceptions_handler import ServiceException

router = APIRouter(tags=["Attack Modules"])


@router.get("/api/v1/attack-modules")
@inject
def get_all_attack_module(
    am_service: AttackModuleService = Depends(Provide[Container.am_service]),
) -> list[str]:
    """
    Retrieve all attack modules from the database.

    Args:
        am_service (AttackModuleService): The service responsible for fetching attack modules.

    Returns:
        list: A list of attack modules if successful.

    Raises:
        HTTPException: An error with status code 404 if attack modules file is not found.
        HTTPException: An error with status code 400 if there is a validation error with the request.
        HTTPException: An error with status code 500 for any other type of server-side error.
    """
    try:
        return am_service.get_all_attack_module()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve attack modules: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve attack modules: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve attack modules: {e.msg}"
            )


@router.get("/api/v1/attack-modules/metadata")
@inject
def get_all_attack_module_metadata(
    am_service: AttackModuleService = Depends(Provide[Container.am_service]),
) -> list:
    try:
        return am_service.get_all_attack_module_metadata()
    except ServiceException as e:
        if e.error_code == "FileNotFound":
            raise HTTPException(
                status_code=404, detail=f"Failed to retrieve attack modules: {e.msg}"
            )
        elif e.error_code == "ValidationError":
            raise HTTPException(
                status_code=400, detail=f"Failed to retrieve attack modules: {e.msg}"
            )
        else:
            raise HTTPException(
                status_code=500, detail=f"Failed to retrieve attack modules: {e.msg}"
            )



from pathlib import Path

from jinja2 import Template

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class PromptTemplate:
    def __init__(self):
        pass

    @staticmethod
    def get_all_prompt_template_names() -> list[str]:
        """
        Retrieve the names of all prompt templates available.

        Returns:
            list: A list of prompt template names.
        """

        filepaths = []
        prompt_template_files = Storage.get_objects(
            EnvVariables.PROMPT_TEMPLATES.name, "json"
        )
        for prompt_template in prompt_template_files:
            filepaths.append(Path(prompt_template).stem)
        return filepaths

    @staticmethod
    def get_all_prompt_template_details() -> list[dict]:
        """
        Retrieve details of all prompt templates available.

        Returns:
            list[dict]: A list of dictionaries containing the details of each prompt template.
        """
        list_of_pt_names = PromptTemplate.get_all_prompt_template_names()
        list_of_pt_contents = []

        for pt_name in list_of_pt_names:
            pt_contents = Storage.read_object(
                EnvVariables.PROMPT_TEMPLATES.name, pt_name, "json"
            )
            id = pt_name
            name = pt_contents["name"]
            description = pt_contents["description"]
            template = pt_contents["template"]
            list_of_pt_contents.append(
                {
                    "id": id,
                    "name": name,
                    "description": description,
                    "template": template,
                }
            )
        return list_of_pt_contents

    @staticmethod
    def delete(pt_id: str) -> bool:
        """
        Deletes a prompt template identified by its unique ID.

        This method attempts to delete the prompt template with the given ID from the storage.
        If the deletion is successful, it returns True. If an exception occurs during the deletion
        process, it prints an error message and re-raises the exception.

        Args:
            pt_id (str): The unique identifier of the prompt template to be deleted.

        Returns:
            bool: True if the prompt template was successfully deleted.

        Raises:
            Exception: If an error occurs during the deletion process.
        """
        try:
            Storage.delete_object(EnvVariables.PROMPT_TEMPLATES.name, pt_id, "json")
            return True

        except Exception as e:
            logger.error(f"Failed to delete prompt template: {str(e)}")
            raise e

    @staticmethod
    def process_prompt_pt(user_prompt: str, prompt_template_name: str) -> str:
        """
        Process a user prompt using a specified prompt template.

        Args:
            user_prompt (str): The user prompt to process.
            prompt_template (str): The name of the prompt template to use.

        Returns:
            str: The processed user prompt based on the template.
        """

        prompt_template_file = Storage.read_object(
            EnvVariables.PROMPT_TEMPLATES.name, prompt_template_name, "json"
        )
        template = prompt_template_file["template"]
        jinja_template = Template(template)
        return jinja_template.render({"prompt": user_prompt})



from typing import Optional

from moonshot.src.recipes.recipe_arguments import RecipeArguments as RecipePydanticModel


class RecipeResponseModel(RecipePydanticModel):
    total_prompt_in_recipe: Optional[int] = None



from ast import literal_eval

import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_delete_attack_module, api_get_all_attack_module_metadata
from moonshot.integrations.cli.utils.process_data import filter_data

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_attack_modules(args) -> list | None:
    """
    Retrieves and prints the metadata of all attack modules.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find attack module(s) with a keyword.
        pagination (str): Optional field to paginate attack modules.

    Returns:
         list | None: A list of AttackModule or None if there is no result.
    """
    try:
        print("Listing attack modules may take a while...")
        attack_module_metadata_list = api_get_all_attack_module_metadata()
        keyword = args.find.lower() if args.find else ""
        pagination = literal_eval(args.pagination) if args.pagination else ()

        if attack_module_metadata_list:
            filtered_attack_modules_list = filter_data(
                attack_module_metadata_list, keyword, pagination
            )
            if filtered_attack_modules_list:
                _display_attack_modules(filtered_attack_modules_list)
                return filtered_attack_modules_list

        console.print("[red]There are no attack modules found.[/red]")
        return None

    except Exception as e:
        print(f"[list_attack_modules]: {str(e)}")


def delete_attack_module(args) -> None:
    """
    Deletes an attack module after confirming with the user.

    Args:
        args (object): The arguments object. It should have a 'attack_modulee' attribute
                       which is the ID of the attack module to delete.
    """
    # Confirm with the user before deleting an attack module
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the attack module (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Attack module deletion cancelled.[/]")
        return
    try:
        api_delete_attack_module(args.attack_module)
        print("[delete_attack_module]: Attack module deleted.")
    except Exception as e:
        print(f"[delete_attack_module]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def _display_attack_modules(attack_modules: list) -> None:
    """
    Display a list of attack modules.

    This function takes a list of attack modules and displays them in a table format. If the list is empty, it prints a
    message indicating that no attack modules were found.

    Args:
        attack_modules (list): A list of attack modules.

    Returns:
        None
    """
    table = Table(
        title="Attack Module List",
        show_lines=True,
        expand=True,
        header_style="bold",
    )
    table.add_column("No.", width=2)
    table.add_column("Details", justify="left", width=98)

    for idx, attack_module_data in enumerate(attack_modules, 1):
        attack_module_data_str = ""
        for k, v in attack_module_data.items():
            if k != "idx":
                attack_module_data_str += f"[blue]{k.capitalize()}:[/blue] {v}\n\n"
        idx = attack_module_data.get("idx", idx)
        table.add_row(str(idx), attack_module_data_str)
    console.print(table)


# Delete attack module arguments
delete_attack_module_args = cmd2.Cmd2ArgumentParser(
    description="Delete an attack module.",
    epilog="Example:\n delete_attack_module sample_attack_module",
)

delete_attack_module_args.add_argument(
    "attack_module", type=str, help="The ID of the attack module to delete"
)

# List attack modules arguments
list_attack_modules_args = cmd2.Cmd2ArgumentParser(
    description="List all attack modules.",
    epilog='Example:\n list_attack_modules -f "text"',
)

list_attack_modules_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find attack module(s) with keyword",
    nargs="?",
)

list_attack_modules_args.add_argument(
    "-p",
    "--pagination",
    type=str,
    help="Optional tuple to paginate attack module(s). E.g. (2,10) returns 2nd page with 10 items in each page.",
    nargs="?",
)



from pydantic import validate_call

from moonshot.src.metrics.metric import Metric


# ------------------------------------------------------------------------------
# Metrics APIs
# ------------------------------------------------------------------------------
@validate_call
def api_delete_metric(met_id: str) -> bool:
    """
    Deletes a metric identified by its unique metric ID.

    Args:
        met_id (str): The unique identifier for the metric to be deleted.

    Returns:
        bool: True if the metric was successfully deleted.

    Raises:
        Exception: If the deletion process encounters an error.
    """
    return Metric.delete(met_id)


def api_get_all_metric() -> list[dict]:
    """
    Retrieves all available metrics.

    This function calls the get_available_items method from the Metric class to retrieve all available metrics.
    It then returns a list of dictionaries, each containing the details of a metric.

    Returns:
        list[dict]: A list of dictionaries, each representing a metric's details.
    """
    _, metrics_info = Metric.get_available_items()
    return metrics_info


def api_get_all_metric_name() -> list[str]:
    """
    Retrieves all available metric names.

    This function calls the get_available_items method from the Metric class to retrieve all available metrics.
    It then extracts the names of each metric and returns a list of these names.

    Returns:
        list[str]: A list of strings, each representing a metric name.
    """
    metrics_names, _ = Metric.get_available_items()
    return metrics_names



from operator import itemgetter
from typing import Any

from pydantic import validate_call

from moonshot.src.api.api_runner import api_get_all_runner, api_load_runner
from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.redteaming.session.session import Session
from moonshot.src.runners.runner_type import RunnerType
from moonshot.src.storage.db_interface import DBInterface
from moonshot.src.storage.storage import Storage


# ------------------------------------------------------------------------------
# Session and Chat APIs
# ------------------------------------------------------------------------------
@validate_call
def api_load_session(runner_id: str) -> dict | None:
    """
    Loads the session details for a specific runner.

    This function calls the `Session.load` method to retrieve the session details associated with the
    specified runner ID.

    Args:
        runner_id (str): The unique identifier of the runner for which the session details are to be loaded.

    Returns:
        dict | None: A dictionary containing the session details if available, otherwise None.
    """
    return Session.load(api_load_runner(runner_id).database_instance)


@validate_call
def api_create_session(
    runner_id: str, database_instance: Any, endpoints: list[str], runner_args: dict
) -> Session:
    """
    Creates a new session for a specific runner.

    This function creates a new session by calling the `Session` constructor with the provided arguments.

    Args:
        runner_id (str): The unique identifier of the runner for which the session is to be created.
        database_instance (Any): The database instance to be used for the session.
        endpoints (list): A list of endpoints for the session.
        runner_args (dict): A dictionary of arguments for the runner.

    Returns:
        Session
    """
    if isinstance(database_instance, DBInterface):
        if runner_id:
            session_instance = Session(
                runner_id,
                RunnerType.REDTEAM,
                {**runner_args},
                database_instance,
                endpoints,
                Storage.get_filepath(
                    EnvVariables.RESULTS.name, runner_id, "json", True
                ),
            )
            return session_instance
        else:
            raise RuntimeError(
                "[Session] Failed to initialise Session. String should have at least 1 character."
            )
    else:
        raise RuntimeError(
            "[Session] Failed to initialise Session. No database instance provided."
        )


def api_get_all_session_names() -> list[str]:
    """
    Retrieves a list of all session names.

    This function calls the `api_get_available_session_info` method to obtain the available session information
    and returns a list of session names.

    Returns:
        list[str]: A list of strings, each denoting a session name.
    """
    session_names, _ = api_get_available_session_info()
    return session_names


def api_get_available_session_info() -> tuple[list, list]:
    """
    Retrieves the IDs and database instances of runners with active sessions.

    This function retrieves the IDs and database instances of runners with active sessions by querying all runners
    and checking if each runner has an active session. It returns a tuple containing a list of runner IDs and a list
    of corresponding session metadata for runners with active sessions.

    Returns:
        tuple[list[str], list[str]]: A tuple containing a list of runner IDs and a list of corresponding session
        metadata for runners with active sessions.
    """
    runners_info = api_get_all_runner()
    runner_instances = [
        api_load_runner(str(runner_info.get("id"))) for runner_info in runners_info
    ]

    runner_ids = []
    session_metadata_list = []

    for runner_instance in runner_instances:
        if runner_instance.database_instance:
            session_metadata = Session.load(runner_instance.database_instance)
            if session_metadata is not None:
                runner_ids.append(runner_instance.id)
                session_metadata_list.append(session_metadata)
    return runner_ids, session_metadata_list


def api_get_all_session_metadata() -> list[dict]:
    """
    Retrieves metadata for all sessions.

    This function retrieves the metadata for all active sessions by calling the `api_get_available_session_info` method.

    Returns:
        list: A list containing the metadata for all active sessions, sorted by created datetime in descending order.
    """
    _, session_metadata_list = api_get_available_session_info()
    return sorted(
        session_metadata_list, key=itemgetter("created_datetime"), reverse=True
    )


@validate_call
def api_update_context_strategy(runner_id: str, context_strategy: str) -> bool:
    """
    Updates the context strategy for a specific runner.

    This function updates the context strategy for a specific runner identified by the given runner_id. It calls the
    `Session.update_context_strategy` method with the runner's database instance,
    runner_id, and the new context_strategy.

    Args:
        runner_id (str): The ID of the runner for which the context strategy needs to be updated.
        context_strategy (str): The new context strategy to be set for the runner.

    Returns:
        bool: The status on whether the context strategy is updated successfully.
    """
    return Session.update_context_strategy(
        api_load_runner(runner_id).database_instance, runner_id, context_strategy
    )


@validate_call
def api_update_cs_num_of_prev_prompts(runner_id: str, num_of_prev_prompts: int) -> bool:
    """
    Updates the number of previous prompts used in a context strategy for a specific runner.

    This function updates the number of previous prompts used in a context strategy for a specific runner identified by
    the given runner_id. It calls the `Session.update_cs_num_of_prev_prompts` method with the runner's database
    instance, runner_id, and the new num_of_prev_prompts.

    Args:
        runner_id (str): The ID of the runner for which the number of previous prompts needs to be updated.
        num_of_prev_prompts (int): The new number of previous prompts to be set for the runner.

    Returns:
        bool: The status on whether the number of prompts for context strategy is updated successfully.
    """
    return Session.update_cs_num_of_prev_prompts(
        api_load_runner(runner_id).database_instance, runner_id, num_of_prev_prompts
    )


@validate_call
def api_update_prompt_template(runner_id: str, prompt_template: str) -> bool:
    """
    Updates the prompt template for a specific runner.

    This function updates the prompt template for a specific runner identified by the given runner_id. It calls the
    `Session.update_prompt_template` method with the runner's database instance,
    runner_id, and the new prompt_template.

    Args:
        runner_id (str): The ID of the runner for which the prompt template needs to be updated.
        prompt_template (str): The new prompt template to be set for the runner.

    Returns:
        bool: The status on whether the prompt template is updated successfully.
    """
    return Session.update_prompt_template(
        api_load_runner(runner_id).database_instance, runner_id, prompt_template
    )


@validate_call
def api_update_metric(runner_id: str, metric_id: str) -> bool:
    """
    Updates the metric for a specific runner.

    This function updates the metric for a specific runner identified by the given runner_id. It calls the
    `Session.update_metric` method with the runner's database instance,
    runner_id, and the new metric_id.

    Args:
        runner_id (str): The ID of the runner for which the metric needs to be updated.
        metric_id (str): The new metric to be set for the runner.

    Returns:
        bool: The status on whether the metric is updated successfully.
    """
    return Session.update_metric(
        api_load_runner(runner_id).database_instance, runner_id, metric_id
    )


@validate_call
def api_update_system_prompt(runner_id: str, system_prompt: str) -> bool:
    """
    Updates the system prompt for a specific runner.

    This function updates the system prompt for a specific runner identified by the given runner_id. It calls the
    `Session.update_system_prompt` method with the runner's database instance,
    runner_id, and the new system_prompt.

    Args:
        runner_id (str): The ID of the runner for which the system prompt needs to be updated.
        system_prompt (str): The new system prompt to be set for the runner.

    Returns:
        bool: The status on whether the system prompt is updated successfully.
    """
    return Session.update_system_prompt(
        api_load_runner(runner_id).database_instance, runner_id, system_prompt
    )


@validate_call
def api_update_attack_module(runner_id: str, attack_module_id: str) -> bool:
    """
    Updates the attack module for a specific runner.

    This function updates the attack module for a specific runner identified by the given runner_id. It calls the
    `Session.update_attack_module` method with the runner's database instance,
    runner_id, and the new attack_module_id.

    Args:
        runner_id (str): The ID of the runner for which the attack module needs to be updated.
        attack_module_id (str): The new attack module to be set for the runner.

    Returns:
        bool: The status on whether the attack module is updated successfully.
    """
    return Session.update_attack_module(
        api_load_runner(runner_id).database_instance, runner_id, attack_module_id
    )


@validate_call
def api_delete_session(runner_id: str) -> bool:
    """
    Deletes the session for a specific runner.

    This function deletes the session for the runner identified by the given runner_id. It calls the
    `Session.delete` method with the runner's database instance.

    Args:
        runner_id (str): The ID of the runner for which the session needs to be deleted.

    Returns:
        bool: The status on whether the session is deleted successfully.
    """
    return Session.delete(api_load_runner(runner_id).database_instance)


@validate_call
def api_get_all_chats_from_session(runner_id: str) -> dict | None:
    """
    Retrieves all chat messages from a specific session.

    This function retrieves all chat messages from the session associated with the specified runner ID.
    It calls the `Session.get_session_chats` method with the runner's database instance.

    Args:
        runner_id (str): The unique identifier of the runner for which the chat messages are to be retrieved.

    Returns:
        dict | None: A dictionary containing all chat messages if available, otherwise None.
    """
    return Session.get_session_chats(api_load_runner(runner_id).database_instance)



from moonshot.src.configs.env_variables import EnvironmentVars


# ------------------------------------------------------------------------------
# Environment Variables APIs
# ------------------------------------------------------------------------------
def api_set_environment_variables(env_vars: dict) -> None:
    """
    Sets the environment variables for the current session.

    Args:
        env_vars (dict): A dictionary containing the environment variables to set.

    Returns:
        None
    """
    EnvironmentVars.load_env(env_vars)



from .... import api as moonshot_api
from ..schemas.dataset_create_dto import DatasetCreateDTO
from ..schemas.dataset_response_dto import DatasetResponseDTO
from ..services.base_service import BaseService
from ..services.utils.exceptions_handler import exception_handler
from .utils.file_manager import copy_file


class DatasetService(BaseService):
    @exception_handler
    def create_dataset(self, dataset_data: DatasetCreateDTO, method: str) -> str:
        """
        Create a dataset using the specified method.

        Args:
            dataset_data (DatasetCreateDTO): The data required to create the dataset.
            method (str): The method to use for creating the dataset.
                          Supported methods are "hf" and "csv".

        Raises:
            Exception: If an error occurs during dataset creation.
        """
        new_ds_path = moonshot_api.api_create_datasets(
            name=dataset_data.name,
            description=dataset_data.description,
            reference=dataset_data.reference,
            license=dataset_data.license,
            method=method,
            **dataset_data.params,
        )
        return copy_file(new_ds_path)

    @exception_handler
    def get_all_datasets(self) -> list[DatasetResponseDTO]:
        datasets = moonshot_api.api_get_all_datasets()
        return [DatasetResponseDTO.model_validate(dataset) for dataset in datasets]

    @exception_handler
    def get_all_datasets_name(self) -> list[str] | None:
        datasets = moonshot_api.api_get_all_datasets_name()
        return datasets

    @exception_handler
    def delete_dataset(self, dataset_id: str) -> None:
        moonshot_api.api_delete_dataset(dataset_id)



from __future__ import annotations

from pathlib import Path

from pydantic import validate_call
from slugify import slugify

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.datasets.dataset import Dataset
from moonshot.src.recipes.recipe_arguments import RecipeArguments
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Recipe:
    def __init__(self, rec_args: RecipeArguments) -> None:
        self.id = rec_args.id
        self.name = rec_args.name
        self.description = rec_args.description
        self.tags = rec_args.tags
        self.categories = rec_args.categories
        self.datasets = rec_args.datasets
        self.prompt_templates = rec_args.prompt_templates
        self.metrics = rec_args.metrics
        self.grading_scale = rec_args.grading_scale
        self.stats = rec_args.stats

    @classmethod
    def load(cls, rec_id: str) -> Recipe:
        """
        Loads a recipe from persistent storage.

        This method constructs the file path for the recipe's JSON file using the provided recipe ID and the
        predefined recipe directory. It reads the JSON file, deserializes the recipe data, and instantiates a Recipe
        object with the loaded data.

        Args:
            rec_id (str): The unique identifier for the recipe to be loaded.

        Returns:
            Recipe: A Recipe object populated with the data from the recipe's JSON file.
        """
        return cls(Recipe.read(rec_id))

    @staticmethod
    def create(rec_args: RecipeArguments) -> str:
        """
        Creates a new recipe and saves its details in a JSON file.

        This method uses the `rec_args` parameter to generate a unique recipe ID by slugifying the recipe name.
        It then builds a dictionary with the recipe's details and writes this information to a JSON file.
        The JSON file is named after the recipe ID and is stored in the directory specified by
        `EnvironmentVars.RECIPES`.
        If any error occurs during the process, an exception is thrown and the error message is printed.

        Args:
            rec_args (RecipeArguments): An object that holds the necessary details to create a new recipe.

        Returns:
            str: The unique ID of the newly created recipe.

        Raises:
            Exception: If an error occurs during the file writing process or any other operation within the method.
        """
        try:
            rec_id = slugify(rec_args.name, lowercase=True)
            rec_info = {
                "name": rec_args.name,
                "description": rec_args.description,
                "tags": rec_args.tags,
                "categories": rec_args.categories,
                "datasets": rec_args.datasets,
                "prompt_templates": rec_args.prompt_templates,
                "metrics": rec_args.metrics,
                "grading_scale": rec_args.grading_scale,
            }

            # check if the recipe exists
            if Storage.is_object_exists(EnvVariables.RECIPES.name, rec_id, "json"):
                raise RuntimeError(f"Recipe with ID '{rec_id}' already exists.")

            Recipe.check_file_exists(
                EnvVariables.PROMPT_TEMPLATES.name,
                rec_args.prompt_templates,
                "Prompt Template",
                "json",
            )
            Recipe.check_file_exists(
                EnvVariables.DATASETS.name, rec_args.datasets, "Dataset", "json"
            )
            Recipe.check_file_exists(
                EnvVariables.METRICS.name, rec_args.metrics, "Metric", "py"
            )

            # Write as json output
            Storage.create_object(EnvVariables.RECIPES.name, rec_id, rec_info, "json")
            return rec_id

        except Exception as e:
            logger.error(f"Failed to create recipe: {str(e)}")
            raise e

    @staticmethod
    @validate_call
    def read(rec_id: str) -> RecipeArguments:
        """
        Retrieves the details of a specific recipe.

        This static method takes a recipe ID as input, locates the corresponding JSON file within the directory
        specified by `EnvironmentVars.RECIPES`, and constructs a RecipeArguments object that contains the details
        of the recipe.

        Args:
            rec_id (str): The unique identifier for the recipe to be retrieved.

        Returns:
            RecipeArguments: A populated object with the recipe's details.

        Raises:
            Exception: If there is an issue reading the file or during any other part of the process.
        """
        try:
            if not rec_id:
                raise RuntimeError("Recipe ID is empty.")

            recipe_details = Recipe._read_recipe(rec_id, {})
            if not recipe_details:
                raise RuntimeError(f"Recipe with ID '{rec_id}' does not exist.")

            return RecipeArguments(**recipe_details)

        except Exception as e:
            logger.error(f"Failed to read recipe: {str(e)}")
            raise e

    @staticmethod
    def _get_datasets_prompt_counts() -> dict:
        """
        Generates a mapping of dataset IDs to their number of prompts.

        This method reads the cache information from the storage, which contains the number of prompts for each dataset.
        It then creates a dictionary mapping each dataset ID to the corresponding number of prompts.

        Returns:
            dict: A dictionary where keys are dataset IDs and values are the number of prompts for that dataset.
        """
        # Calculate statistics for the recipe and update the results dictionary with them
        _, dataset_results = Dataset.get_available_items()
        # Create a mapping of dataset IDs to their number of prompts
        return {
            dataset.id: dataset.num_of_dataset_prompts for dataset in dataset_results
        }

    @staticmethod
    def _read_recipe(rec_id: str, dataset_prompts_count: dict) -> dict:
        """
        Reads the recipe JSON file based on the provided recipe ID and dataset prompts count
        and returns the recipe information as a dictionary.

        Args:
            rec_id (str): The unique identifier for the recipe.
            dataset_prompts_count (dict): A dictionary mapping dataset IDs to their number of prompts.

        Returns:
            dict: A dictionary containing the recipe information.

        Raises:
            RuntimeError: If the recipe file cannot be read or does not exist.
        """
        obj_results = {"id": rec_id}
        recipe_info = Storage.read_object(EnvVariables.RECIPES.name, rec_id, "json")
        if not recipe_info:
            raise RuntimeError(f"Unable to get results for {rec_id}.")
        else:
            obj_results.update(recipe_info)

        # Calculate statistics for the recipe and update the results dictionary with them
        stats = {
            "num_of_tags": len(obj_results["tags"]),
            "num_of_datasets": len(obj_results["datasets"]),
            "num_of_prompt_templates": len(obj_results["prompt_templates"]),
            "num_of_metrics": len(obj_results["metrics"]),
            "num_of_datasets_prompts": {},
        }

        if dataset_prompts_count:
            stats["num_of_datasets_prompts"] = {
                dataset_name: dataset_prompts_count.get(dataset_name, 0)
                for dataset_name in obj_results["datasets"]
            }
        else:
            _, datasets_metadata = Dataset.get_available_items(obj_results["datasets"])
            stats["num_of_datasets_prompts"] = {
                dataset.id: dataset.num_of_dataset_prompts
                for dataset in datasets_metadata
            }

        obj_results["stats"] = stats
        return obj_results

    @staticmethod
    def update(rec_args: RecipeArguments) -> bool:
        """
        Updates the recipe information in the storage based on the provided RecipeArguments object.

        This method processes an instance of RecipeArguments, transforming it into a dictionary format. It ensures
        that certain required files exist before proceeding to update the recipe information in the storage system.
        Upon successful update, it returns True. If any error is encountered during the process, the error message is
        logged, and the exception is propagated upwards.

        Args:
            rec_args (RecipeArguments): An object containing the updated recipe information.

        Returns:
            bool: Indicates whether the recipe update was successful (True) or not (False).

        Raises:
            Exception: Propagates any exceptions that occur during the update process.
        """
        try:
            # Transform RecipeArguments into a dictionary, excluding the 'id' field
            rec_info = rec_args.to_dict()
            rec_info.pop("id", None)  # The 'id' is derived and should not be written

            # Verify existence of related files in storage
            Recipe.check_file_exists(
                EnvVariables.PROMPT_TEMPLATES.name,
                rec_args.prompt_templates,
                "Prompt Template",
                "json",
            )
            Recipe.check_file_exists(
                EnvVariables.DATASETS.name, rec_args.datasets, "Dataset", "json"
            )
            Recipe.check_file_exists(
                EnvVariables.METRICS.name, rec_args.metrics, "Metric", "py"
            )

            # Persist the updated recipe information to storage
            Storage.create_object(
                EnvVariables.RECIPES.name, rec_args.id, rec_info, "json"
            )
            return True

        except Exception as e:
            logger.error(f"Failed to update recipe: {str(e)}")
            raise e

    @staticmethod
    @validate_call
    def delete(rec_id: str) -> bool:
        """
        Deletes a recipe identified by its unique ID.

        This method attempts to delete the recipe with the given ID from the storage.
        If the deletion is successful, it returns True. If an exception occurs during the deletion
        process, it prints an error message and re-raises the exception.

        Args:
            rec_id (str): The unique identifier of the recipe to be deleted.

        Returns:
            bool: True if the recipe was successfully deleted.

        Raises:
            Exception: If an error occurs during the deletion process.
        """
        try:
            Storage.delete_object(EnvVariables.RECIPES.name, rec_id, "json")
            return True

        except Exception as e:
            logger.error(f"Failed to delete recipe: {str(e)}")
            raise e

    @staticmethod
    def get_available_items() -> tuple[list[str], list[RecipeArguments]]:
        """
        Retrieves a list of available recipe IDs and their corresponding recipe information.

        This method queries the storage for all available recipes and filters out any system files or directories.
        It then creates a list of RecipeArguments objects with detailed information about each recipe and a list of
        their IDs. Finally, it returns a tuple containing the list of recipe IDs and the list of RecipeArguments
        objects.

        Returns:
            tuple[list[str], list[RecipeArguments]]: A tuple containing a list of recipe IDs and a list of
                                                      RecipeArguments objects for each available recipe.

        Raises:
            Exception: If there's an error during the retrieval process.
        """
        try:
            retn_recs = []
            retn_recs_ids = []

            datasets_prompt_counts = Recipe._get_datasets_prompt_counts()
            recs = Storage.get_objects(EnvVariables.RECIPES.name, "json")
            for rec in recs:
                if "__" in rec:
                    continue

                rec_info = RecipeArguments(
                    **Recipe._read_recipe(Path(rec).stem, datasets_prompt_counts)
                )
                retn_recs.append(rec_info)
                retn_recs_ids.append(rec_info.id)

            return retn_recs_ids, retn_recs

        except Exception as e:
            logger.error(f"Failed to get available recipes: {str(e)}")
            raise e

    @staticmethod
    def check_file_exists(
        env_var_name: str, file_list: list, file_type: str, extension: str
    ) -> None:
        """
        Checks if a specified file exists in the storage.

        Args:
            env_var_name (str): The environment variable name where the file is stored.
            file_name (str): The name of the file to check.
            file_type (str): The type of the file.
            extension (str): The extension of the file.

        Raises:
            RuntimeError: If the file does not exist in the storage.
        """
        for file_name in file_list:
            if file_name and not Storage.is_object_exists(
                env_var_name, file_name, extension
            ):
                raise RuntimeError(f"[Recipe] {file_type} {file_name} does not exist.")



from __future__ import annotations

from pathlib import Path

import pandas as pd
from datasets import load_dataset
from pydantic import validate_call
from slugify import slugify

from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.datasets.dataset_arguments import DatasetArguments
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Dataset:
    cache_name = "cache"
    cache_extension = "json"

    @staticmethod
    @validate_call
    def create(ds_args: DatasetArguments, method: str, **kwargs) -> str:
        """
        Creates a new dataset based on the provided arguments and method.

        This method generates a unique dataset ID using the dataset name,
        checks if a dataset with the same ID already exists, and then
        creates the dataset using the specified method (either 'csv' or
        'hf'). The dataset information is then stored as a JSON object.

        Args:
            ds_args (DatasetArguments): The arguments containing dataset
                details such as name, description, reference, and license.
            method (str): The method to create the dataset. It can be either
                'csv' or 'hf'.
            **kwargs: Additional keyword arguments required for the specified
                method.
                - For 'csv' method: 'csv_file_path' (str): The file path to
                    the CSV file.
                - For 'hf' method: 'dataset_name' (str): The name of the
                    Hugging Face dataset.
                    'dataset_config' (str): The configuration of the Hugging
                    Face dataset.
                    'split' (str): The split of the dataset to load.
                    'input_col' (list[str]): The list of input columns.
                    'target_col' (str): The target column.

        Returns:
            str: The unique ID of the created dataset.

        Raises:
            RuntimeError: If a dataset with the same ID already exists.
            Exception: If any other error occurs during the dataset creation
                process.
        """
        try:
            ds_id = slugify(ds_args.name, lowercase=True)

            # Check if the dataset exists
            if Storage.is_object_exists(EnvVariables.DATASETS.name, ds_id, "json"):
                raise RuntimeError(f"Dataset with ID '{ds_id}' already exists.")

            examples = [{}]
            if method == "csv":
                examples = Dataset._convert_csv(kwargs["csv_file_path"])
            elif method == "hf":
                examples = Dataset._download_hf(kwargs)

            ds_info = {
                "id": ds_id,
                "name": ds_args.name,
                "description": ds_args.description,
                "reference": ds_args.reference,
                "license": ds_args.license,
                "examples": examples,
            }

            # Write as JSON output
            file_path = Storage.create_object(
                EnvVariables.DATASETS.name, ds_id, ds_info, "json"
            )
            return file_path

        except Exception as e:
            logger.error(f"Failed to create dataset: {str(e)}")
            raise e

    @staticmethod
    def _convert_csv(csv_file: str) -> list[dict]:
        """
        Converts a CSV file to a list of dictionaries.

        This method reads a CSV file and converts its contents into a list of dictionaries,
        where each dictionary represents a row in the CSV file.

        Args:
            csv_file (str): The file path to the CSV file.

        Returns:
            list[dict]: A list of dictionaries representing the CSV data.
        """
        df = pd.read_csv(csv_file)
        data = df.to_dict("records")
        return data

    @staticmethod
    def _download_hf(hf_args) -> list[dict]:
        """
        Downloads a dataset from Hugging Face and converts it to a list of dictionaries.

        This method loads a dataset from Hugging Face based on the provided arguments and converts
        its contents into a list of dictionaries, where each dictionary contains 'input' and 'target' keys.

        Args:
            hf_args (dict): A dictionary containing the following keys:
                - 'dataset_name' (str): The name of the Hugging Face dataset.
                - 'dataset_config' (str): The configuration of the Hugging Face dataset.
                - 'split' (str): The split of the dataset to load.
                - 'input_col' (list[str]): The list of input columns.
                - 'target_col' (str): The target column.

        Returns:
            list[dict]: A list of dictionaries representing the dataset.
        """
        dataset = load_dataset(hf_args["dataset_name"], hf_args["dataset_config"])
        data = []
        for example in dataset[hf_args["split"]]:
            input_data = " ".join([str(example[col]) for col in hf_args["input_col"]])
            target_data = str(example[hf_args["target_col"]])
            data.append({"input": input_data, "target": target_data})
        return data

    @staticmethod
    @validate_call
    def read(ds_id: str) -> DatasetArguments:
        """
        Fetches the details of a given dataset.

        This method takes a dataset ID as input, finds the corresponding JSON file in the directory
        specified by `EnvVariables.DATASETS`, and returns a DatasetArguments object
        that contains the dataset's details. If any error arises during the process, an exception is raised and the
        error message is logged.

        Args:
            ds_id (str): The unique ID of the dataset to be fetched.

        Returns:
            DatasetArguments: An object encapsulating the details of the fetched dataset.

        Raises:
            Exception: If there's an error during the file reading process or any other operation within the method.
        """
        try:
            if ds_id:
                return DatasetArguments(**Dataset._read_dataset(ds_id))
            else:
                raise RuntimeError("Dataset ID is empty")

        except Exception as e:
            logger.error(f"Failed to read dataset: {str(e)}")
            raise e

    @staticmethod
    def _read_dataset(ds_id: str) -> dict:
        """
        Retrieves dataset information from storage and augments it with metadata.

        This method takes a dataset ID, locates the corresponding JSON file within the directory
        specified by `EnvVariables.DATASETS`, and constructs a dictionary that includes the dataset's
        core details, as well as metadata such as the creation datetime and the count of dataset prompts.

        Args:
            ds_id (str): The unique identifier of the dataset to be retrieved.

        Returns:
            dict: A dictionary with the dataset's core information, enriched with metadata like the creation datetime
                  and the total number of prompts contained within the dataset.
        """
        # Read the basic dataset information
        dataset_info = Storage.read_object_with_iterator(
            obj_type=EnvVariables.DATASETS.name,
            obj_id=ds_id,
            obj_extension="json",
            json_keys=["name", "description", "license", "reference"],
            iterator_keys=["examples.item"],
        )

        # Add additional parameters - [id, num_of_dataset_prompts, creation_date]
        # Append the dataset ID to the dataset_info
        dataset_info["id"] = ds_id

        # Use Storage.count_objects to get the number of examples in a memory-efficient way
        dataset_info["num_of_dataset_prompts"] = Storage.count_objects(
            EnvVariables.DATASETS.name, ds_id, "json", "examples.item"
        )

        # Assign the creation date to the dataset_info
        creation_datetime = Storage.get_creation_datetime(
            EnvVariables.DATASETS.name, ds_id, "json"
        )
        dataset_info["created_date"] = creation_datetime.replace(
            microsecond=0
        ).isoformat(" ")

        return dataset_info

    @staticmethod
    @validate_call
    def delete(ds_id: str) -> bool:
        """
        Deletes a dataset from storage.

        This method attempts to delete the dataset with the given ID from the storage. If the deletion is successful,
        it returns True. If an exception occurs during the deletion process, it prints an error message and re-raises
        the exception.

        Args:
            ds_id (str): The unique identifier of the dataset to be deleted.

        Returns:
            bool: True if the dataset was successfully deleted.

        Raises:
            Exception: If an error occurs during the deletion process.
        """
        try:
            Storage.delete_object(EnvVariables.DATASETS.name, ds_id, "json")
            return True

        except Exception as e:
            logger.error(f"Failed to delete dataset: {str(e)}")
            raise e

    @staticmethod
    def get_cache_information() -> dict:
        """
        Retrieves cache information from the storage.

        This method attempts to read the cache information from the storage and return it as a dictionary.
        If the cache information does not exist or an error occurs, it returns an empty dictionary.

        Returns:
            dict: A dictionary containing the cache information or an empty dictionary if an error occurs
            or if the cache information does not exist.

        Raises:
            Exception: If there's an error during the retrieval process, it is logged and an
            empty dictionary is returned.
        """
        try:
            # Retrieve cache information from the storage and return it as a dictionary
            cache_info = Storage.read_object(
                EnvVariables.DATASETS.name, Dataset.cache_name, Dataset.cache_extension
            )
            return cache_info if cache_info else {}
        except Exception as e:
            logger.error(f"Failed to retrieve cache information: {str(e)}")
            return {}

    @staticmethod
    def write_cache_information(cache_info: dict) -> None:
        """
        Writes the updated cache information to the storage.

        Args:
            cache_info (dict): The cache information to be written.
        """
        try:
            Storage.create_object(
                obj_type=EnvVariables.DATASETS.name,
                obj_id=Dataset.cache_name,
                obj_info=cache_info,
                obj_extension=Dataset.cache_extension,
            )
        except Exception as e:
            logger.error(f"Failed to write cache information: {str(e)}")
            raise e

    @staticmethod
    def get_available_items(
        datasets: list[str] = [],
    ) -> tuple[list[str], list[DatasetArguments]]:
        """
        Retrieves a list of available dataset IDs and their corresponding DatasetArguments objects.

        This method filters out any non-dataset files and the cache file from the list of datasets. It then
        retrieves or updates the dataset information from the cache for each dataset. If the cache is updated
        during this process, it writes the updated cache information back to the storage.

        Args:
            datasets (list[str], optional): A list of dataset file names. If not provided, it will retrieve
                the list of all dataset files from the storage. Defaults to an empty list.

        Returns:
            tuple[list[str], list[DatasetArguments]]: A tuple containing two lists:
                - The first list contains the IDs of the available datasets.
                - The second list contains the corresponding DatasetArguments objects for those IDs.
        """
        try:
            retn_datasets = []
            retn_datasets_ids = []
            ds_cache_info = Dataset.get_cache_information()
            cache_needs_update = False  # Initialize a flag to track cache updates

            if datasets:
                datasets_objects = datasets
            else:
                datasets_objects = Storage.get_objects(
                    EnvVariables.DATASETS.name, "json"
                )

            for ds in datasets_objects:
                if (
                    "__" in ds
                    or f"{Dataset.cache_name}.{Dataset.cache_extension}" in ds
                ):
                    continue

                ds_name = Path(ds).stem
                ds_info, cache_updated = Dataset._get_or_update_dataset_info(
                    ds_name, ds_cache_info
                )
                if cache_updated:
                    cache_needs_update = True  # Set the flag if any cache was updated

                retn_datasets.append(ds_info)
                retn_datasets_ids.append(ds_info.id)

            if cache_needs_update:  # Check the flag after the loop
                Dataset.write_cache_information(ds_cache_info)

            return retn_datasets_ids, retn_datasets

        except Exception as e:
            logger.error(f"Failed to get available datasets: {str(e)}")
            raise e

    @staticmethod
    def _get_or_update_dataset_info(
        ds_name: str, ds_cache_info: dict
    ) -> tuple[DatasetArguments, bool]:
        """
        Retrieves or updates the dataset information from the cache.

        This method checks if the dataset information is already available in the cache and if the file hash matches
        the one stored in the cache. If it does, the information is retrieved from the cache. If not, the dataset
        information is read from the storage, the cache is updated with the new information and the new file hash,
        and a flag is set to indicate that the cache has been updated.

        Args:
            ds_name (str): The name of the dataset.
            ds_cache_info (dict): A dictionary containing the cached dataset information.

        Returns:
            tuple[DatasetArguments, bool]: A tuple containing the DatasetArguments object with the dataset information
                                           and a boolean indicating whether the cache was updated or not.
        """
        file_hash = Storage.get_file_hash(EnvVariables.DATASETS.name, ds_name, "json")
        cache_updated = False

        if ds_name in ds_cache_info and file_hash == ds_cache_info[ds_name]["hash"]:
            ds_metadata = ds_cache_info[ds_name].copy()
            ds_metadata.pop("hash", None)
            ds_info = DatasetArguments(**ds_metadata)
        else:
            ds_info = DatasetArguments(**Dataset._read_dataset(ds_name))
            ds_info.examples = None
            ds_cache_info[ds_name] = ds_info.copy().to_dict()
            ds_cache_info[ds_name]["hash"] = file_hash
            cache_updated = True

        return ds_info, cache_updated



import argparse

import cmd2

from moonshot.integrations.cli.common.dataset import (
    add_dataset,
    add_dataset_args
)
from moonshot.integrations.cli.common.connectors import (
    add_endpoint,
    add_endpoint_args,
    delete_endpoint,
    delete_endpoint_args,
    list_connector_types,
    list_connector_types_args,
    list_endpoints,
    list_endpoints_args,
    update_endpoint,
    update_endpoint_args,
    view_endpoint,
    view_endpoint_args,
)
from moonshot.integrations.cli.common.prompt_template import (
    delete_prompt_template,
    delete_prompt_template_args,
    list_prompt_templates,
    list_prompt_templates_args,
)


@cmd2.with_default_category("Moonshot Common")
class CommonCommandSet(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    # ------------------------------------------------------------------------------
    # List contents
    # ------------------------------------------------------------------------------

    @cmd2.with_argparser(list_connector_types_args)
    def do_list_connector_types(self, args: argparse.Namespace) -> None:
        list_connector_types(args)

    @cmd2.with_argparser(list_endpoints_args)
    def do_list_endpoints(self, args: argparse.Namespace) -> None:
        list_endpoints(args)

    @cmd2.with_argparser(list_prompt_templates_args)
    def do_list_prompt_templates(self, args: argparse.Namespace) -> None:
        list_prompt_templates(args)

    @cmd2.with_argparser(delete_prompt_template_args)
    def do_delete_prompt_template(self, args: argparse.Namespace) -> None:
        delete_prompt_template(args)

    # ------------------------------------------------------------------------------
    # Add contents
    # ------------------------------------------------------------------------------
    @cmd2.with_argparser(add_endpoint_args)
    def do_add_endpoint(self, args: argparse.Namespace) -> None:
        add_endpoint(args)

    @cmd2.with_argparser(add_dataset_args)
    def do_add_dataset(self, args:argparse.Namespace) -> None:
        add_dataset(args)
        
    # ------------------------------------------------------------------------------
    # Delete contents
    # ------------------------------------------------------------------------------
    @cmd2.with_argparser(delete_endpoint_args)
    def do_delete_endpoint(self, args: argparse.Namespace) -> None:
        delete_endpoint(args)

    # ------------------------------------------------------------------------------
    # Update contents
    # ------------------------------------------------------------------------------
    @cmd2.with_argparser(update_endpoint_args)
    def do_update_endpoint(self, args: argparse.Namespace) -> None:
        update_endpoint(args)

    # ------------------------------------------------------------------------------
    # View contents
    # ------------------------------------------------------------------------------
    @cmd2.with_argparser(view_endpoint_args)
    def do_view_endpoint(self, args: argparse.Namespace) -> None:
        view_endpoint(args)



from typing import Any

from .... import api as moonshot_api
from ..services.utils.exceptions_handler import exception_handler
from .base_service import BaseService


class PromptTemplateService(BaseService):
    @exception_handler
    def get_prompt_templates(self) -> list[dict[str, Any]]:
        """
        Retrieve all prompt templates with their details.

        Returns:
            list[dict[str, Any]]: A list of dictionaries containing the details of each prompt template.
        """
        templates = moonshot_api.api_get_all_prompt_template_detail()
        return templates

    @exception_handler
    def get_prompt_templates_name(self) -> list[str]:
        """
        Retrieve the names of all prompt templates.

        Returns:
            list[str]: A list of names of all prompt templates.
        """
        templates = moonshot_api.api_get_all_prompt_template_name()
        return templates

    @exception_handler
    def delete_prompt_template(self, pt_id: str):
        """
        Delete a prompt template by its identifier.

        Args:
            pt_id (str): The unique identifier of the prompt template to be deleted.
        """
        moonshot_api.api_delete_prompt_template(pt_id)
